@@ -59,6 +59,10 @@ function cleanup_manila {
     _clean_share_group $SHARE_GROUP $SHARE_NAME_PREFIX
     _clean_manila_lvm_backing_file $SHARE_GROUP
     _clean_zfsonlinux_data
+
+    if [ $(trueorfalse False MANILA_USE_UWSGI) == True ]; then
+        remove_uwsgi_config "$MANILA_UWSGI_CONF" "$MANILA_WSGI"
+    fi
 }
 
 # _config_manila_apache_wsgi() - Configure manila-api wsgi application.
@@ -71,7 +75,7 @@ function _config_manila_apache_wsgi {
     sudo sed -e "
         s|%APACHE_NAME%|$APACHE_NAME|g;
         s|%MANILA_BIN_DIR%|$MANILA_BIN_DIR|g;
-        s|%PORT%|$MANILA_SERVICE_PORT|g;
+        s|%PORT%|$REAL_MANILA_SERVICE_PORT|g;
         s|%APIWORKERS%|$API_WORKERS|g;
         s|%USER%|$STACK_USER|g;
     " -i $manila_api_apache_conf
@@ -102,6 +106,10 @@ function configure_default_backends {
         if [ $(trueorfalse False MANILA_USE_SERVICE_INSTANCE_PASSWORD) == True ]; then
             iniset $MANILA_CONF $group_name service_instance_password $MANILA_SERVICE_INSTANCE_PASSWORD
         fi
+
+        if [ "$SHARE_DRIVER" == "manila.share.drivers.generic.GenericShareDriver" ]; then
+            iniset $MANILA_CONF $group_name ssh_conn_timeout $MANILA_SSH_TIMEOUT
+        fi
     done
 }
 
@@ -140,6 +148,23 @@ function set_cinder_quotas {
     fi
 }
 
+function set_backend_availability_zones {
+    ENABLED_BACKENDS=$1
+    echo_summary "Setting up backend_availability_zone option \
+        for any enabled backends that do not use the Generic driver. \
+        Availability zones for the Generic driver must coincide with those \
+        created for Nova and Cinder."
+    local zonenum
+    generic_driver='manila.share.drivers.generic.GenericShareDriver'
+    for BE in ${ENABLED_BACKENDS//,/ }; do
+        share_driver=$(iniget $MANILA_CONF $BE share_driver)
+        if [[ $share_driver != $generic_driver ]]; then
+            zone="manila-zone-$((zonenum++))"
+            iniset $MANILA_CONF $BE backend_availability_zone $zone
+        fi
+    done
+}
+
 # configure_manila - Set config files, create data dirs, etc
 function configure_manila {
     if [[ ! -d $MANILA_CONF_DIR ]]; then
@@ -188,7 +213,7 @@ function configure_manila {
     # Remove old conf file if exists
     rm -f $MANILA_CONF
 
-    configure_auth_token_middleware $MANILA_CONF manila  $MANILA_AUTH_CACHE_DIR
+    configure_keystone_authtoken_middleware $MANILA_CONF manila
 
     iniset $MANILA_CONF DEFAULT auth_strategy keystone
     iniset $MANILA_CONF DEFAULT debug True
@@ -200,13 +225,22 @@ function configure_manila {
     iniset $MANILA_CONF DEFAULT rootwrap_config $MANILA_CONF_DIR/rootwrap.conf
     iniset $MANILA_CONF DEFAULT osapi_share_extension manila.api.contrib.standard_extensions
     iniset $MANILA_CONF DEFAULT state_path $MANILA_STATE_PATH
-    iniset $MANILA_CONF DEFAULT default_share_type $MANILA_DEFAULT_SHARE_TYPE
-    iniset $MANILA_CONF DEFAULT default_share_group_type $MANILA_DEFAULT_SHARE_GROUP_TYPE
+
+    # Note: Sample share types will still be created if the below is False
+
+    if [ $(trueorfalse False MANILA_CONFIGURE_DEFAULT_TYPES) == True ]; then
+        iniset $MANILA_CONF DEFAULT default_share_type $MANILA_DEFAULT_SHARE_TYPE
+        iniset $MANILA_CONF DEFAULT default_share_group_type $MANILA_DEFAULT_SHARE_GROUP_TYPE
+    fi
 
     if ! [[ -z $MANILA_SHARE_MIGRATION_PERIOD_TASK_INTERVAL ]]; then
         iniset $MANILA_CONF DEFAULT migration_driver_continue_update_interval $MANILA_SHARE_MIGRATION_PERIOD_TASK_INTERVAL
     fi
 
+    if ! [[ -z $MANILA_SERVER_MIGRATION_PERIOD_TASK_INTERVAL ]]; then
+        iniset $MANILA_CONF DEFAULT server_migration_driver_continue_update_interval $MANILA_SERVER_MIGRATION_PERIOD_TASK_INTERVAL
+    fi
+
     if ! [[ -z $MANILA_DATA_COPY_CHECK_HASH ]]; then
         iniset $MANILA_CONF DEFAULT check_hash $MANILA_DATA_COPY_CHECK_HASH
     fi
@@ -221,17 +255,21 @@ function configure_manila {
 
     # Set the replica_state_update_interval
     iniset $MANILA_CONF DEFAULT replica_state_update_interval $MANILA_REPLICA_STATE_UPDATE_INTERVAL
+    # Set the use_scheduler_creating_share_from_snapshot
+    iniset $MANILA_CONF DEFAULT use_scheduler_creating_share_from_snapshot $MANILA_USE_SCHEDULER_CREATING_SHARE_FROM_SNAPSHOT
 
     if is_service_enabled neutron; then
-        configure_auth_token_middleware $MANILA_CONF neutron $MANILA_AUTH_CACHE_DIR neutron
+        configure_keystone_authtoken_middleware $MANILA_CONF neutron neutron
     fi
     if is_service_enabled nova; then
-        configure_auth_token_middleware $MANILA_CONF nova $MANILA_AUTH_CACHE_DIR nova
+        configure_keystone_authtoken_middleware $MANILA_CONF nova nova
     fi
     if is_service_enabled cinder; then
-        configure_auth_token_middleware $MANILA_CONF cinder $MANILA_AUTH_CACHE_DIR cinder
+        configure_keystone_authtoken_middleware $MANILA_CONF cinder cinder
+    fi
+    if is_service_enabled glance; then
+        configure_keystone_authtoken_middleware $MANILA_CONF glance glance
     fi
-
     # Note: set up config group does not mean that this backend will be enabled.
     # To enable it, specify its name explicitly using "enabled_share_backends" opt.
     configure_default_backends
@@ -251,32 +289,29 @@ function configure_manila {
 
     iniset $MANILA_CONF DEFAULT manila_service_keypair_name $MANILA_SERVICE_KEYPAIR_NAME
 
+    REAL_MANILA_SERVICE_PORT=$MANILA_SERVICE_PORT
     if is_service_enabled tls-proxy; then
+        # Set the protocol to 'https', update the endpoint base and set the default port
+        MANILA_SERVICE_PROTOCOL="https"
+        MANILA_ENDPOINT_BASE="${MANILA_ENDPOINT_BASE/http:/https:}"
+        REAL_MANILA_SERVICE_PORT=$MANILA_SERVICE_PORT_INT
         # Set the service port for a proxy to take the original
-        iniset $MANILA_CONF DEFAULT osapi_share_listen_port $MANILA_SERVICE_PORT_INT
-    fi
-
-    if [ "$SYSLOG" != "False" ]; then
-        iniset $MANILA_CONF DEFAULT use_syslog True
+        iniset $MANILA_CONF DEFAULT osapi_share_listen_port $REAL_MANILA_SERVICE_PORT
+        iniset $MANILA_CONF oslo_middleware enable_proxy_headers_parsing True
     fi
 
     iniset_rpc_backend manila $MANILA_CONF DEFAULT
 
-    if [ "$LOG_COLOR" == "True" ] && [ "$SYSLOG" == "False" ]; then
-        # Add color to logging output
-        iniset $MANILA_CONF DEFAULT logging_context_format_string \
-            "%(asctime)s.%(msecs)d %(color)s%(levelname)s %(name)s [[01;36m%(request_id)s [00;36m%(user_id)s %(project_id)s%(color)s] [01;35m%(instance)s%(color)s%(message)s[00m"
-        iniset $MANILA_CONF DEFAULT logging_default_format_string \
-            "%(asctime)s.%(msecs)d %(color)s%(levelname)s %(name)s [[00;36m-%(color)s] [01;35m%(instance)s%(color)s%(message)s[00m"
-        iniset $MANILA_CONF DEFAULT logging_debug_format_suffix \
-            "[00;33mfrom (pid=%(process)d) %(funcName)s %(pathname)s:%(lineno)d[00m"
-        iniset $MANILA_CONF DEFAULT logging_exception_prefix \
-            "%(color)s%(asctime)s.%(msecs)d TRACE %(name)s [01;35m%(instance)s[00m"
-    fi
+    setup_logging $MANILA_CONF
 
     MANILA_CONFIGURE_GROUPS=${MANILA_CONFIGURE_GROUPS:-"$MANILA_ENABLED_BACKENDS"}
     set_config_opts $MANILA_CONFIGURE_GROUPS
     set_config_opts DEFAULT
+    set_backend_availability_zones $MANILA_ENABLED_BACKENDS
+
+    if [ $(trueorfalse False MANILA_USE_UWSGI) == True ]; then
+        write_uwsgi_config "$MANILA_UWSGI_CONF" "$MANILA_WSGI" "/share"
+    fi
 
     if [ $(trueorfalse False MANILA_USE_MOD_WSGI) == True ]; then
         _config_manila_apache_wsgi
@@ -332,13 +367,7 @@ function create_service_share_servers {
                 fi
 
                 floating_ip=$(openstack floating ip create $PUBLIC_NETWORK_NAME --subnet $PUBLIC_SUBNET_NAME | grep 'floating_ip_address' | get_field 2)
-                # TODO(rishabh-d-dave): For time being circumvent the bug -
-                # https://bugs.launchpad.net/python-openstackclient/+bug/1747721
-                # Once fixed, replace the following 3 lines by -
-                # openstack server add floating ip $vm_id $floating_ip
-                vm_port_id=$(openstack port list --server $vm_id -c ID -f \
-                    value)
-                openstack floating ip set --port $vm_port_id $floating_ip
+                openstack server add floating ip $vm_id $floating_ip
 
                 iniset $MANILA_CONF $BE service_instance_name_or_id $vm_id
                 iniset $MANILA_CONF $BE service_net_name_or_ip $floating_ip
@@ -378,10 +407,10 @@ function configure_data_service_generic_driver {
     if [[ $share_driver == $generic_driver ]]; then
         driver_handles_share_servers=$(iniget $MANILA_CONF ${enabled_backends[0]} driver_handles_share_servers)
         if [[ $(trueorfalse False driver_handles_share_servers) == False ]]; then
-            iniset $MANILA_CONF DEFAULT data_node_access_ip $PUBLIC_NETWORK_GATEWAY
+            iniset $MANILA_CONF DEFAULT data_node_access_ips $PUBLIC_NETWORK_GATEWAY
         else
             if ! [[ -z $MANILA_DATA_NODE_IP ]]; then
-                iniset $MANILA_CONF DEFAULT data_node_access_ip $MANILA_DATA_NODE_IP
+                iniset $MANILA_CONF DEFAULT data_node_access_ips $MANILA_DATA_NODE_IP
             fi
         fi
     fi
@@ -406,14 +435,12 @@ function create_manila_service_flavor {
 # create_manila_service_image - creates image, that will be used by backends
 # with configured generic driver to boot Nova VMs from.
 function create_manila_service_image {
-    if is_service_enabled nova; then
+    if is_service_enabled nova g-api; then
         TOKEN=$(openstack token issue -c id -f value)
         local image_exists=$( openstack image list | grep " $MANILA_SERVICE_IMAGE_NAME " )
         if [[ -z $image_exists ]]; then
             # Download Manila's image
-            if is_service_enabled g-reg; then
-                upload_image $MANILA_SERVICE_IMAGE_URL $TOKEN
-            fi
+            upload_image $MANILA_SERVICE_IMAGE_URL $TOKEN
         fi
     fi
 }
@@ -465,19 +492,14 @@ function create_manila_accounts {
 
     create_service_user "manila"
 
-    # Set up Manila v1 service and endpoint
     get_or_create_service "manila" "share" "Manila Shared Filesystem Service"
     get_or_create_endpoint "share" "$REGION_NAME" \
-        "$MANILA_SERVICE_PROTOCOL://$MANILA_SERVICE_HOST:$MANILA_SERVICE_PORT/v1/\$(tenant_id)s" \
-        "$MANILA_SERVICE_PROTOCOL://$MANILA_SERVICE_HOST:$MANILA_SERVICE_PORT/v1/\$(tenant_id)s" \
-        "$MANILA_SERVICE_PROTOCOL://$MANILA_SERVICE_HOST:$MANILA_SERVICE_PORT/v1/\$(tenant_id)s"
+        "$MANILA_ENDPOINT_BASE/v1/\$(project_id)s"
 
     # Set up Manila v2 service and endpoint
     get_or_create_service "manilav2" "sharev2" "Manila Shared Filesystem Service V2"
     get_or_create_endpoint "sharev2" "$REGION_NAME" \
-        "$MANILA_SERVICE_PROTOCOL://$MANILA_SERVICE_HOST:$MANILA_SERVICE_PORT/v2/\$(tenant_id)s" \
-        "$MANILA_SERVICE_PROTOCOL://$MANILA_SERVICE_HOST:$MANILA_SERVICE_PORT/v2/\$(tenant_id)s" \
-        "$MANILA_SERVICE_PROTOCOL://$MANILA_SERVICE_HOST:$MANILA_SERVICE_PORT/v2/\$(tenant_id)s"
+        "$MANILA_ENDPOINT_BASE/v2/\$(project_id)s"
 }
 
 # create_default_share_group_type - create share group type that will be set as default.
@@ -491,13 +513,11 @@ function create_default_share_group_type {
     fi
 }
 
-# create_default_share_type - create share type that will be set as default.
+# create_default_share_type - create share type that will be set as default
+# if $MANILA_CONFIGURE_DEFAULT_TYPES is set to True, if set to False, the share
+# type identified by $MANILA_DEFAULT_SHARE_TYPE is still created, but not
+# configured as default.
 function create_default_share_type {
-    echo "Waiting for Manila API to start..."
-    if ! wait_for_service 60 $MANILA_SERVICE_PROTOCOL://$MANILA_SERVICE_HOST:$MANILA_SERVICE_PORT; then
-        die $LINENO "Manila did not start"
-    fi
-
     enabled_backends=(${MANILA_ENABLED_BACKENDS//,/ })
     driver_handles_share_servers=$(iniget $MANILA_CONF ${enabled_backends[0]} driver_handles_share_servers)
 
@@ -532,6 +552,7 @@ function create_custom_share_types {
 
 # configure_backing_file - Set up backing file for LVM
 function configure_backing_file {
+    sudo vgscan
     if ! sudo vgs $SHARE_GROUP; then
         if [ "$CONFIGURE_BACKING_FILE" = "True" ]; then
             SHARE_BACKING_FILE=${SHARE_BACKING_FILE:-$DATA_DIR/${SHARE_GROUP}-backing-file}
@@ -637,15 +658,10 @@ function init_manila {
                 cat $STACK_HOME/.ssh/*.pub >> $SSH_USER_HOME/.ssh/authorized_keys
                 # Give ssh user sudo access
                 echo "$MANILA_ZFSONLINUX_SSH_USERNAME ALL=(ALL) NOPASSWD: ALL" | sudo tee -a /etc/sudoers > /dev/null
-                iniset $MANILA_CONF DEFAULT data_node_access_ip $MANILA_ZFSONLINUX_SERVICE_IP
+                iniset $MANILA_CONF DEFAULT data_node_access_ips $MANILA_ZFSONLINUX_SERVICE_IP
             fi
         fi
     fi
-
-    # Create cache dir
-    sudo mkdir -p $MANILA_AUTH_CACHE_DIR
-    sudo chown $STACK_USER $MANILA_AUTH_CACHE_DIR
-    rm -f $MANILA_AUTH_CACHE_DIR/*
 }
 
 # check_nfs_kernel_service_state_ubuntu- Make sure nfsd is running
@@ -700,8 +716,10 @@ function install_manila {
 
     if is_service_enabled m-shr; then
 
-        if [[ ! $(systemctl is-active nfs-ganesha.service) == 'active' ]]; then
-            _install_nfs_and_samba
+        if [[ ! $(systemctl is-active nfs-ganesha.service) == 'active' ]] ; then
+            if [ "$SHARE_DRIVER" != "manila.share.drivers.cephfs.driver.CephFSDriver" ] ; then
+                _install_nfs_and_samba
+            fi
         fi
 
         if [ "$SHARE_DRIVER" == "manila.share.drivers.zfsonlinux.driver.ZFSonLinuxShareDriver" ]; then
@@ -724,11 +742,12 @@ function install_manila {
                         sudo apt-get install -y linux-headers-generic
                         sudo apt-get install -y build-essential
                         sudo apt-get install -y ubuntu-zfs
-                    elif [[ $(lsb_release -s -d) == *"16.04"* ]]; then
-                        # Xenial
+
+                    elif [[ $(echo $(lsb_release -rs) '>=' 16.04 | bc -l) == 1 ]]; then
+                        # Xenial and beyond
                         sudo apt-get install -y zfsutils-linux
                     else
-                        echo "Only 'Trusty' and 'Xenial' releases of Ubuntu are supported."
+                        echo "Only 'Trusty', 'Xenial' and newer releases of Ubuntu are supported."
                         exit 1
                     fi
                 else
@@ -743,6 +762,9 @@ function install_manila {
             fi
             check_nfs_kernel_service_state_ubuntu
         elif [ "$SHARE_DRIVER" == $MANILA_CONTAINER_DRIVER ]; then
+            # Remove workaround for https://launchpad.net/bugs/1876820 when
+            # container driver stops using "brctl"
+            install_package bridge-utils
             if is_ubuntu; then
                 echo "Installing docker...."
                 install_docker_ubuntu
@@ -793,29 +815,63 @@ function configure_samba {
             iniset $MANILA_CONF $backend_name driver_handles_share_servers False
             iniset $MANILA_CONF $backend_name lvm_share_export_ips $MANILA_LVM_SHARE_EXPORT_IPS
         done
-        iniset $MANILA_CONF DEFAULT data_node_access_ip $HOST_IP
+        iniset $MANILA_CONF DEFAULT data_node_access_ips $HOST_IP
     fi
 }
 
 # start_manila_api - starts manila API services and checks its availability
 function start_manila_api {
-    if [ $(trueorfalse False MANILA_USE_MOD_WSGI) == True ]; then
+
+    # NOTE(vkmc) If both options are set to true we are using uwsgi
+    # as the preferred way to deploy manila. See
+    # https://governance.openstack.org/tc/goals/pike/deploy-api-in-wsgi.html#uwsgi-vs-mod-wsgi
+    # for more details
+    if [ $(trueorfalse False MANILA_USE_UWSGI) == True ] && [ $(trueorfalse False MANILA_USE_MOD_WSGI) == True ]; then
+        MSG="Both MANILA_USE_UWSGI and MANILA_USE_MOD_WSGI are set to True.
+            Using UWSGI as the preferred option
+            Set MANILA_USE_UWSGI to False to deploy manila api with MOD_WSGI"
+        warn $LINENO $MSG
+    fi
+
+    if [ $(trueorfalse False MANILA_USE_UWSGI) == True ]; then
+        echo "Deploying with UWSGI"
+        run_process m-api "$(which uwsgi) --ini $MANILA_UWSGI_CONF --procname-prefix manila-api"
+    elif [ $(trueorfalse False MANILA_USE_MOD_WSGI) == True ]; then
+        echo "Deploying with MOD_WSGI"
         install_apache_wsgi
         enable_apache_site manila-api
         restart_apache_server
         tail_log m-api /var/log/$APACHE_NAME/manila_api.log
     else
+        echo "Deploying with built-in server"
         run_process m-api "$MANILA_BIN_DIR/manila-api --config-file $MANILA_CONF"
     fi
 
     echo "Waiting for Manila API to start..."
-    if ! wait_for_service $SERVICE_TIMEOUT $MANILA_SERVICE_PROTOCOL://$MANILA_SERVICE_HOST:$MANILA_SERVICE_PORT; then
+    # This is a health check against the manila-api service we just started.
+    # We use the port ($REAL_MANILA_SERVICE_PORT) here because we want to hit
+    # the bare service endpoint, even if the tls tunnel should be enabled.
+    # We're making sure that the internal port is checked using unencryted
+    # traffic at this point.
+
+    local MANILA_HEALTH_CHECK_URL=$MANILA_SERVICE_PROTOCOL://$MANILA_SERVICE_HOST:$REAL_MANILA_SERVICE_PORT
+
+    if [ $(trueorfalse False MANILA_USE_UWSGI) == True ]; then
+        MANILA_HEALTH_CHECK_URL=$MANILA_ENDPOINT_BASE
+    fi
+
+    if ! wait_for_service $SERVICE_TIMEOUT $MANILA_HEALTH_CHECK_URL; then
         die $LINENO "Manila API did not start"
     fi
 
     # Start proxies if enabled
-    if is_service_enabled tls-proxy; then
-        start_tls_proxy '*' $MANILA_SERVICE_PORT $MANILA_SERVICE_HOST $MANILA_SERVICE_PORT_INT &
+    #
+    # If tls-proxy is enabled and MANILA_USE_UWSGI is set to True, a generic
+    # http-services-tls-proxy will be set up to handle tls-termination to
+    # manila as well as all the other https services, we don't need to
+    # create our own.
+    if [ $(trueorfalse False MANILA_USE_UWSGI) == False ] && is_service_enabled tls-proxy; then
+        start_tls_proxy manila '*' $MANILA_SERVICE_PORT $MANILA_SERVICE_HOST $MANILA_SERVICE_PORT_INT
     fi
 }
 
@@ -850,7 +906,7 @@ function stop_manila {
 }
 
 function install_manila_tempest_plugin {
-    MANILA_TEMPEST_PLUGIN_REPO=${MANILA_TEMPEST_PLUGIN_REPO:-${GIT_BASE}/openstack/manila-tempest-plugin.git}
+    MANILA_TEMPEST_PLUGIN_REPO=${MANILA_TEMPEST_PLUGIN_REPO:-${GIT_BASE}/openstack/manila-tempest-plugin}
     MANILA_TEMPEST_PLUGIN_BRANCH=${MANILA_TEMPEST_PLUGIN_BRANCH:-master}
     MANILA_TEMPEST_PLUGIN_DIR=$DEST/manila-tempest-plugin
 
@@ -861,6 +917,14 @@ function install_manila_tempest_plugin {
 # update_tempest - Function used for updating Tempest config if Tempest service enabled
 function update_tempest {
     if is_service_enabled tempest; then
+
+        if [[ "$(trueorfalse False MANILA_SETUP_IPV6)" == "True" ]]; then
+            # The public network was created by us, so set it explicitly in
+            # tempest.conf
+            public_net_id=$(openstack network list --name $PUBLIC_NETWORK_NAME -f value -c ID )
+            iniset $TEMPEST_CONFIG network public_network_id $public_net_id
+        fi
+
         TEMPEST_CONFIG=${TEMPEST_CONFIG:-$TEMPEST_DIR/etc/tempest.conf}
         ADMIN_TENANT_NAME=${ADMIN_TENANT_NAME:-"admin"}
         ADMIN_DOMAIN_NAME=${ADMIN_DOMAIN_NAME:-"Default"}
@@ -955,10 +1019,41 @@ function install_libraries {
     fi
 }
 
+function allow_host_ports_for_share_mounting {
+
+    if [[ $MANILA_ENABLED_SHARE_PROTOCOLS =~ NFS ]]; then
+        # 111 and 2049 are for rpcbind and NFS
+        # Other ports are for NFSv3 statd, mountd and lockd daemons
+        MANILA_TCP_PORTS=(2049 111 32803 892 875 662)
+        MANILA_UDP_PORTS=(111 32769 892 875 662)
+    fi
+    if [[ $MANILA_ENABLED_SHARE_PROTOCOLS =~ CEPHFS ]]; then
+        # clients need access to the ceph daemons
+        MANILA_TCP_PORTS=(${MANILA_TCP_PORTS[*]} 6789 6800:7300)
+    fi
+
+    if [[ -v MANILA_TCP_PORTS || -v MANILA_UDP_PORTS ]]; then
+        for ipcmd in iptables ip6tables; do
+            sudo $ipcmd -N manila-storage
+            sudo $ipcmd -I INPUT 1 -j manila-storage
+            for port in ${MANILA_TCP_PORTS[*]}; do
+                sudo $ipcmd -A manila-storage -m tcp -p tcp --dport $port -j ACCEPT
+            done
+            for port in ${MANILA_UDP_PORTS[*]}; do
+                sudo $ipcmd -A manila-storage -m udp -p udp --dport $port -j ACCEPT
+            done
+        done
+    fi
+}
+
 function setup_ipv6 {
 
-    # save IPv6 default route to add back later after enabling forwarding
-    local default_route=$(ip -6 route | grep default | cut -d ' ' -f1,2,3,4,5)
+    # This will fail with multiple default routes and is not needed in CI
+    # but may be useful when developing with devstack locally
+    if [ $(trueorfalse False MANILA_RESTORE_IPV6_DEFAULT_ROUTE) == True ]; then
+        # save IPv6 default route to add back later after enabling forwarding
+        local default_route=$(ip -6 route | grep default | cut -d ' ' -f1,2,3,4,5)
+    fi
 
     # make sure those system values are set
     sudo sysctl -w net.ipv6.conf.lo.disable_ipv6=0
@@ -969,8 +1064,8 @@ function setup_ipv6 {
     sudo ovs-vsctl set Bridge $PUBLIC_BRIDGE other_config:disable-in-band=true
 
     # Create address scopes and subnet pools
-    neutron address-scope-create --shared scope-v4 4
-    neutron address-scope-create --shared scope-v6 6
+    openstack address scope create --share --ip-version 4 scope-v4
+    openstack address scope create --share --ip-version 6 scope-v6
     openstack subnet pool create $SUBNETPOOL_NAME_V4 --default-prefix-length $SUBNETPOOL_SIZE_V4 --pool-prefix $SUBNETPOOL_PREFIX_V4 --address-scope scope-v4 --default --share
     openstack subnet pool create $SUBNETPOOL_NAME_V6 --default-prefix-length $SUBNETPOOL_SIZE_V6 --pool-prefix $SUBNETPOOL_PREFIX_V6 --address-scope scope-v6 --default --share
 
@@ -1002,7 +1097,14 @@ function setup_ipv6 {
         for backend_name in ${MANILA_ENABLED_BACKENDS//,/ }; do
             iniset $MANILA_CONF $backend_name lvm_share_export_ips $public_gateway_ipv4,$public_gateway_ipv6
         done
-        iniset $MANILA_CONF DEFAULT data_node_access_ip $public_gateway_ipv4
+        iniset $MANILA_CONF DEFAULT data_node_access_ips $public_gateway_ipv4
+    fi
+
+    if [ "$SHARE_DRIVER" == "manila.share.drivers.cephfs.driver.CephFSDriver" ]; then
+        for backend_name in ${MANILA_ENABLED_BACKENDS//,/ }; do
+            iniset $MANILA_CONF $backend_name cephfs_ganesha_export_ips $public_gateway_ipv4,$public_gateway_ipv6
+        done
+        iniset $MANILA_CONF DEFAULT data_node_access_ips $public_gateway_ipv4
     fi
 
     # install Quagga for setting up the host routes dynamically
@@ -1027,16 +1129,22 @@ function setup_ipv6 {
     echo "log file /var/log/quagga/zebra.log"
     ) | sudo tee /etc/quagga/zebra.conf > /dev/null
 
+    # set Quagga vtysh.conf
+    (
+    echo "service integrated-vtysh-config"
+    echo "username quagga nopassword"
+    ) | sudo tee /etc/quagga/vtysh.conf > /dev/null
+
     # set Quagga bgpd.conf
     (
     echo "log file /var/log/quagga/bgpd.log"
     echo "bgp multiple-instance"
     echo "router bgp 200"
     echo " bgp router-id 1.2.3.4"
-    echo " neighbor ::1 remote-as 100"
-    echo " neighbor ::1 passive"
+    echo " neighbor $public_gateway_ipv6 remote-as 100"
+    echo " neighbor $public_gateway_ipv6 passive"
     echo " address-family ipv6"
-    echo "  neighbor ::1 activate"
+    echo "  neighbor $public_gateway_ipv6 activate"
     echo "line vty"
     echo "debug bgp events"
     echo "debug bgp filters"
@@ -1045,23 +1153,48 @@ function setup_ipv6 {
     echo "debug bgp updates"
     ) | sudo tee /etc/quagga/bgpd.conf > /dev/null
 
-    if is_ubuntu; then
-        sudo systemctl enable quagga
-        sudo systemctl restart quagga
-    else
+    # Quagga logging
+    sudo mkdir -p /var/log/quagga
+    sudo touch /var/log/quagga/zebra.log
+    sudo touch /var/log/quagga/bgpd.log
+    sudo chown -R quagga:quagga /var/log/quagga
+
+
+    GetOSVersion
+    QUAGGA_SERVICES="zebra bgpd"
+    if [[ is_ubuntu && "$os_CODENAME" == "xenial" ]]; then
+        # In Ubuntu Xenial, the services bgpd and zebra are under
+        # one systemd unit: quagga
+        QUAGGA_SERVICES="quagga"
+    elif is_fedora; then
         # Disable SELinux rule that conflicts with Zebra
         sudo setsebool -P zebra_write_config 1
-        sudo systemctl enable zebra
-        sudo systemctl enable bgpd
-        sudo systemctl restart zebra
-        sudo systemctl restart bgpd
+    fi
+    sudo systemctl enable $QUAGGA_SERVICES
+    sudo systemctl restart $QUAGGA_SERVICES
+
+    # log the systemd status
+    sudo systemctl status $QUAGGA_SERVICES
+
+    # This will fail with mutltiple default routes and is not needed in CI
+    # but may be useful when developing with devstack locally
+    if [ $(trueorfalse False MANILA_RESTORE_IPV6_DEFAULT_ROUTE) == True ]; then
+        # add default IPv6 route back
+        if ! [[ -z $default_route ]]; then
+            # "replace" should ignore "RTNETLINK answers: File exists"
+            # error if the route wasn't flushed by the bgp setup we did earlier.
+            sudo ip -6 route replace $default_route
+        fi
     fi
 
-    # add default IPv6 route back
-    if ! [[ -z $default_route ]]; then
-        sudo ip -6 route add $default_route
-    fi
+}
 
+function setup_bgp_for_ipv6 {
+    public_gateway_ipv6=$(openstack subnet show ipv6-public-subnet -c gateway_ip -f value)
+    neutron bgp-speaker-create --ip-version 6 --local-as 100 bgpspeaker
+    neutron bgp-speaker-network-add bgpspeaker $PUBLIC_NETWORK_NAME
+    neutron bgp-peer-create --peer-ip $public_gateway_ipv6 --remote-as 200 bgppeer
+    neutron bgp-speaker-peer-add bgpspeaker bgppeer
 }
 
 # Main dispatcher
@@ -1141,7 +1274,7 @@ elif [[ "$1" == "stack" && "$2" == "extra" ]]; then
     create_custom_share_types
 
     echo_summary "Manila UI is no longer enabled by default. \
-        Add enable_plugin manila-ui github.com/openstack/manila-ui \
+        Add enable_plugin manila-ui https://opendev.org/openstack/manila-ui \
         to your local.conf file to enable Manila UI"
 
 elif [[ "$1" == "stack" && "$2" == "test-config" ]]; then
@@ -1162,10 +1295,32 @@ elif [[ "$1" == "stack" && "$2" == "test-config" ]]; then
     fi
     ###########################################################################
 
-    echo_summary "Fetching and installing manila-tempest-plugin system-wide"
-    install_manila_tempest_plugin
+
+    if [ $(trueorfalse False MANILA_INSTALL_TEMPEST_PLUGIN_SYSTEMWIDE) == True ]; then
+        echo_summary "Fetching and installing manila-tempest-plugin system-wide"
+        install_manila_tempest_plugin
+        export DEPRECATED_TEXT="$DEPRECATED_TEXT\nInstalling
+        manila-tempest-plugin can be done with the help of its own DevStack
+        plugin by adding: \n\n\t'enable_plugin manila-tempest-plugin
+        https://opendev.org/openstack/manila-tempest-plugin'.\n\nManila's
+        DevStack plugin will stop installing it automatically."
+    fi
+
     echo_summary "Update Tempest config"
     update_tempest
+
+
+    if [[ "$(trueorfalse False MANILA_ALLOW_NAS_SERVER_PORTS_ON_HOST)" == "True" ]]; then
+        echo_summary "Allowing IPv4 and IPv6 access to NAS ports on the host"
+        allow_host_ports_for_share_mounting
+    fi
+
+    if [[ "$(trueorfalse False MANILA_SETUP_IPV6)" == "True" ]]; then
+        # Now that all plugins are loaded, setup BGP
+        echo_summary "Setting up BGP speaker to advertise routes to project networks"
+        setup_bgp_for_ipv6
+    fi
+
 fi
 
 if [[ "$1" == "unstack" ]]; then
