@@ -30,7 +30,7 @@ LOG = log.getLogger(__name__)
 
 class APIMapper(routes.Mapper):
     def routematch(self, url=None, environ=None):
-        if url is "":
+        if url == "":
             result = self._match("", environ)
             return result[0], result[1]
         return routes.Mapper.routematch(self, url, environ)
@@ -73,6 +73,7 @@ class APIRouter(base_wsgi.Router):
     def __init__(self, ext_mgr=None):
         if ext_mgr is None:
             if self.ExtensionManager:
+                # pylint: disable=not-callable
                 ext_mgr = self.ExtensionManager()
             else:
                 raise Exception(_("Must specify an ExtensionManager class"))
