@@ -14,7 +14,6 @@
 #    under the License.
 
 from oslo_log import log
-import six
 from six.moves import http_client
 import webob
 from webob import exc
@@ -140,7 +139,7 @@ class ShareController(shares.ShareMixin,
                         'state': constants.ACCESS_STATE_ACTIVE
                     }
                     msg = _("Snapshot %(snap_id)s belongs to a share "
-                            "%(share_id)s which has access rules that are"
+                            "%(share_id)s which has access rules that are "
                             "not %(state)s.")
                     raise exc.HTTPConflict(explanation=msg % msg_args)
 
@@ -150,13 +149,13 @@ class ShareController(shares.ShareMixin,
 
             self.share_api.revert_to_snapshot(context, share, snapshot)
         except exception.ShareNotFound as e:
-            raise exc.HTTPNotFound(explanation=six.text_type(e))
+            raise exc.HTTPNotFound(explanation=e)
         except exception.ShareSnapshotNotFound as e:
-            raise exc.HTTPBadRequest(explanation=six.text_type(e))
+            raise exc.HTTPBadRequest(explanation=e)
         except exception.ShareSizeExceedsAvailableQuota as e:
-            raise exc.HTTPForbidden(explanation=six.text_type(e))
+            raise exc.HTTPForbidden(explanation=e)
         except exception.ReplicationException as e:
-            raise exc.HTTPBadRequest(explanation=six.text_type(e))
+            raise exc.HTTPBadRequest(explanation=e)
 
         return webob.Response(status_int=http_client.ACCEPTED)
 
@@ -178,19 +177,25 @@ class ShareController(shares.ShareMixin,
 
         return data
 
-    @wsgi.Controller.api_version("2.31")
+    @wsgi.Controller.api_version("2.48")
     def create(self, req, body):
+        return self._create(req, body,
+                            check_create_share_from_snapshot_support=True,
+                            check_availability_zones_extra_spec=True)
+
+    @wsgi.Controller.api_version("2.31", "2.47")  # noqa
+    def create(self, req, body):  # pylint: disable=function-redefined  # noqa F811
         return self._create(
             req, body, check_create_share_from_snapshot_support=True)
 
     @wsgi.Controller.api_version("2.24", "2.30")  # noqa
-    def create(self, req, body):  # pylint: disable=E0102
+    def create(self, req, body):  # pylint: disable=function-redefined  # noqa F811
         body.get('share', {}).pop('share_group_id', None)
         return self._create(req, body,
                             check_create_share_from_snapshot_support=True)
 
     @wsgi.Controller.api_version("2.0", "2.23")  # noqa
-    def create(self, req, body):  # pylint: disable=E0102
+    def create(self, req, body):  # pylint: disable=function-redefined  # noqa F811
         body.get('share', {}).pop('share_group_id', None)
         return self._create(req, body)
 
@@ -272,7 +277,7 @@ class ShareController(shares.ShareMixin,
                 new_share_network=new_share_network,
                 new_share_type=new_share_type)
         except exception.Conflict as e:
-            raise exc.HTTPConflict(explanation=six.text_type(e))
+            raise exc.HTTPConflict(explanation=e)
 
         return webob.Response(status_int=return_code)
 
@@ -401,18 +406,29 @@ class ShareController(shares.ShareMixin,
     @wsgi.Controller.api_version('2.7', '2.7')
     def manage(self, req, body):
         body.get('share', {}).pop('is_public', None)
-        detail = self._manage(req, body)
+        detail = self._manage(req, body, allow_dhss_true=False)
         return detail
 
-    @wsgi.Controller.api_version("2.8")  # noqa
-    def manage(self, req, body):  # pylint: disable=E0102
-        detail = self._manage(req, body)
+    @wsgi.Controller.api_version("2.8", "2.48")  # noqa
+    def manage(self, req, body):  # pylint: disable=function-redefined  # noqa F811
+        detail = self._manage(req, body, allow_dhss_true=False)
         return detail
 
-    @wsgi.Controller.api_version('2.7')
+    @wsgi.Controller.api_version("2.49")  # noqa
+    def manage(self, req, body):  # pylint: disable=function-redefined  # noqa F811
+        detail = self._manage(req, body, allow_dhss_true=True)
+        return detail
+
+    @wsgi.Controller.api_version('2.7', '2.48')
     @wsgi.action('unmanage')
     def unmanage(self, req, id, body=None):
-        return self._unmanage(req, id, body)
+        return self._unmanage(req, id, body, allow_dhss_true=False)
+
+    @wsgi.Controller.api_version('2.49')  # noqa
+    @wsgi.action('unmanage')
+    def unmanage(self, req, id,  # pylint: disable=function-redefined # noqa F811
+                 body=None):
+        return self._unmanage(req, id, body, allow_dhss_true=True)
 
     @wsgi.Controller.api_version('2.27')
     @wsgi.action('revert')
