@@ -81,14 +81,6 @@ class Service(BASE, ManilaBase):
     )
 
 
-class ManilaNode(BASE, ManilaBase):
-    """Represents a running manila service on a host."""
-
-    __tablename__ = 'manila_nodes'
-    id = Column(Integer, primary_key=True)
-    service_id = Column(Integer, ForeignKey('services.id'), nullable=True)
-
-
 class Quota(BASE, ManilaBase):
     """Represents a single quota override for a project.
 
@@ -257,6 +249,11 @@ class Share(BASE, ManilaBase):
             return len(replicas) > 1
         return False
 
+    @property
+    def progress(self):
+        if len(self.instances) > 0:
+            return self.instance.progress
+
     @property
     def instance(self):
         # NOTE(gouthamr): The order of preference: status 'replication_change',
@@ -267,8 +264,10 @@ class Share(BASE, ManilaBase):
         if len(self.instances) > 0:
             order = (constants.STATUS_REVERTING,
                      constants.STATUS_REPLICATION_CHANGE,
-                     constants.STATUS_MIGRATING, constants.STATUS_AVAILABLE,
-                     constants.STATUS_ERROR)
+                     constants.STATUS_MIGRATING,
+                     constants.STATUS_SERVER_MIGRATING,
+                     constants.STATUS_AVAILABLE,
+                     constants.STATUS_ERROR, )
             other_statuses = (
                 [x['status'] for x in self.instances if
                  x['status'] not in order and
@@ -318,7 +317,7 @@ class Share(BASE, ManilaBase):
     task_state = Column(String(255))
     instances = orm.relationship(
         "ShareInstance",
-        lazy='immediate',
+        lazy='subquery',
         primaryjoin=(
             'and_('
             'Share.id == ShareInstance.share_id, '
@@ -364,6 +363,7 @@ class ShareInstance(BASE, ManilaBase):
     deleted = Column(String(36), default='False')
     host = Column(String(255))
     status = Column(String(255))
+    progress = Column(String(32))
 
     ACCESS_STATUS_PRIORITIES = {
         constants.STATUS_ACTIVE: 0,
@@ -388,7 +388,7 @@ class ShareInstance(BASE, ManilaBase):
                                   nullable=True)
     _availability_zone = orm.relationship(
         "AvailabilityZone",
-        lazy='immediate',
+        lazy='subquery',
         foreign_keys=availability_zone_id,
         primaryjoin=(
             'and_('
@@ -400,7 +400,8 @@ class ShareInstance(BASE, ManilaBase):
 
     export_locations = orm.relationship(
         "ShareInstanceExportLocations",
-        lazy='immediate',
+        lazy='joined',
+        backref=orm.backref('share_instance', lazy='joined'),
         primaryjoin=(
             'and_('
             'ShareInstance.id == '
@@ -414,7 +415,7 @@ class ShareInstance(BASE, ManilaBase):
                              nullable=True)
     share_type = orm.relationship(
         "ShareTypes",
-        lazy='immediate',
+        lazy='subquery',
         foreign_keys=share_type_id,
         primaryjoin='and_('
                     'ShareInstance.share_type_id == ShareTypes.id, '
@@ -430,10 +431,14 @@ class ShareInstanceExportLocations(BASE, ManilaBase):
     @property
     def el_metadata(self):
         el_metadata = {}
-        for meta in self._el_metadata_bare:  # pylint: disable=E1101
+        for meta in self._el_metadata_bare:  # pylint: disable=no-member
             el_metadata[meta['key']] = meta['value']
         return el_metadata
 
+    @property
+    def replica_state(self):
+        return self.share_instance['replica_state']
+
     id = Column(Integer, primary_key=True)
     uuid = Column(String(36), nullable=False, unique=True)
     share_instance_id = Column(
@@ -467,7 +472,7 @@ class ShareInstanceExportLocationsMetadata(BASE, ManilaBase):
 
     @property
     def export_location_uuid(self):
-        return self.export_location.uuid  # pylint: disable=E1101
+        return self.export_location.uuid  # pylint: disable=no-member
 
 
 class ShareTypes(BASE, ManilaBase):
@@ -668,8 +673,9 @@ class ShareSnapshot(BASE, ManilaBase):
                 lambda x: qualified_replica(x.share_instance), self.instances))
 
             migrating_snapshots = list(filter(
-                lambda x: x.share_instance['status'] ==
-                constants.STATUS_MIGRATING, self.instances))
+                lambda x: x.share_instance['status'] in (
+                    constants.STATUS_MIGRATING,
+                    constants.STATUS_SERVER_MIGRATING), self.instances))
 
             snapshot_instances = (replica_snapshots or migrating_snapshots
                                   or self.instances)
@@ -701,6 +707,7 @@ class ShareSnapshot(BASE, ManilaBase):
 
         order = (constants.STATUS_DELETING, constants.STATUS_CREATING,
                  constants.STATUS_ERROR, constants.STATUS_MIGRATING,
+                 constants.STATUS_SERVER_MIGRATING,
                  constants.STATUS_AVAILABLE)
         other_statuses = [x['status'] for x in self.instances if
                           x['status'] not in order]
@@ -746,6 +753,14 @@ class ShareSnapshotInstance(BASE, ManilaBase):
         # with share drivers
         return self.share_instance_id
 
+    @property
+    def size(self):
+        # NOTE(silvacarlose) for backwards compatibility
+        if self.instance_size is None:
+            return self.snapshot.size
+        else:
+            return self.instance_size
+
     id = Column(String(36), primary_key=True)
     deleted = Column(String(36), default='False')
     snapshot_id = Column(String(36), nullable=True)
@@ -755,7 +770,7 @@ class ShareSnapshotInstance(BASE, ManilaBase):
     progress = Column(String(255))
     provider_location = Column(String(255))
     share_proto = Column(String(255))
-    size = Column(Integer)
+    instance_size = Column('size', Integer)
     share_group_snapshot_id = Column(String(36), nullable=True)
     user_id = Column(String(255))
     project_id = Column(String(255))
@@ -903,14 +918,6 @@ class ShareNetwork(BASE, ManilaBase):
     deleted = Column(String(36), default='False')
     project_id = Column(String(255), nullable=False)
     user_id = Column(String(255), nullable=False)
-    neutron_net_id = Column(String(36), nullable=True)
-    neutron_subnet_id = Column(String(36), nullable=True)
-    network_type = Column(String(32), nullable=True)
-    segmentation_id = Column(Integer, nullable=True)
-    cidr = Column(String(64), nullable=True)
-    gateway = Column(String(64), nullable=True)
-    mtu = Column(Integer, nullable=True)
-    ip_version = Column(Integer, nullable=True)
     name = Column(String(255), nullable=True)
     description = Column(String(255), nullable=True)
     security_services = orm.relationship(
@@ -932,24 +939,89 @@ class ShareNetwork(BASE, ManilaBase):
         primaryjoin='and_('
                     'ShareNetwork.id == ShareInstance.share_network_id,'
                     'ShareInstance.deleted == "False")')
+    share_network_subnets = orm.relationship(
+        "ShareNetworkSubnet",
+        lazy='joined',
+        backref=orm.backref('share_network', lazy='joined'),
+        primaryjoin='and_'
+                    '(ShareNetwork.id == ShareNetworkSubnet.share_network_id,'
+                    'ShareNetworkSubnet.deleted == "False")')
+
+
+class ShareNetworkSubnet(BASE, ManilaBase):
+    """Represents a share network subnet used by some resources."""
+
+    _extra_keys = ['availability_zone']
+
+    __tablename__ = 'share_network_subnets'
+    id = Column(String(36), primary_key=True, nullable=False)
+    neutron_net_id = Column(String(36), nullable=True)
+    neutron_subnet_id = Column(String(36), nullable=True)
+    network_type = Column(String(32), nullable=True)
+    cidr = Column(String(64), nullable=True)
+    segmentation_id = Column(Integer, nullable=True)
+    gateway = Column(String(64), nullable=True)
+    mtu = Column(Integer, nullable=True)
+    deleted = Column(String(36), default='False')
+    share_network_id = Column(String(36), ForeignKey('share_networks.id'),
+                              nullable=False)
+    ip_version = Column(Integer, nullable=True)
+    availability_zone_id = Column(
+        String(36), ForeignKey('availability_zones.id'), nullable=True)
+
     share_servers = orm.relationship(
-        "ShareServer", backref='share_network',
-        primaryjoin='and_(ShareNetwork.id == ShareServer.share_network_id,'
+        "ShareServer", backref='share_network_subnet',
+        lazy='immediate',
+        primaryjoin='and_(ShareNetworkSubnet.id '
+                    '== ShareServer.share_network_subnet_id,'
                     'ShareServer.deleted == "False")')
 
+    _availability_zone = orm.relationship(
+        "AvailabilityZone",
+        lazy='immediate',
+        foreign_keys=availability_zone_id,
+        primaryjoin=(
+            "and_("
+            "ShareNetworkSubnet.availability_zone_id == AvailabilityZone.id, "
+            "AvailabilityZone.deleted == 'False')"))
+
+    @property
+    def availability_zone(self):
+        if self._availability_zone:
+            return self._availability_zone['name']
+
+    @property
+    def is_default(self):
+        return self.availability_zone_id is None
+
+    @property
+    def share_network_name(self):
+        return self.share_network['name']
+
 
 class ShareServer(BASE, ManilaBase):
     """Represents share server used by share."""
     __tablename__ = 'share_servers'
     id = Column(String(36), primary_key=True, nullable=False)
     deleted = Column(String(36), default='False')
-    share_network_id = Column(String(36), ForeignKey('share_networks.id'),
-                              nullable=True)
+    share_network_subnet_id = Column(
+        String(36), ForeignKey('share_network_subnets.id'),
+        nullable=True)
     host = Column(String(255), nullable=False)
-    status = Column(Enum(constants.STATUS_INACTIVE, constants.STATUS_ACTIVE,
-                         constants.STATUS_ERROR, constants.STATUS_DELETING,
-                         constants.STATUS_CREATING, constants.STATUS_DELETED),
-                    default=constants.STATUS_INACTIVE)
+    is_auto_deletable = Column(Boolean, default=True)
+    identifier = Column(String(255), nullable=True)
+    task_state = Column(String(255), nullable=True)
+    source_share_server_id = Column(String(36), ForeignKey('share_servers.id'),
+                                    nullable=True)
+    status = Column(Enum(
+        constants.STATUS_INACTIVE, constants.STATUS_ACTIVE,
+        constants.STATUS_ERROR, constants.STATUS_DELETING,
+        constants.STATUS_CREATING, constants.STATUS_DELETED,
+        constants.STATUS_MANAGING, constants.STATUS_UNMANAGING,
+        constants.STATUS_UNMANAGE_ERROR, constants.STATUS_MANAGE_ERROR,
+        constants.STATUS_SERVER_MIGRATING,
+        constants.STATUS_SERVER_MIGRATING_TO),
+        default=constants.STATUS_INACTIVE)
     network_allocations = orm.relationship(
         "NetworkAllocation",
         primaryjoin='and_('
@@ -1085,10 +1157,10 @@ class ShareGroup(BASE, ManilaBase):
         ShareGroupTypes,
         backref="share_groups",
         foreign_keys=share_group_type_id,
-        primaryjoin='and_('
-                    'ShareGroup.share_group_type_id == '
-                    'ShareGroupTypes.id,'
-                    'ShareGroup.deleted == 0)')
+        primaryjoin="and_("
+                    "ShareGroup.share_group_type_id =="
+                    "ShareGroupTypes.id,"
+                    "ShareGroup.deleted == 'False')")
     _availability_zone = orm.relationship(
         "AvailabilityZone",
         lazy='immediate',
@@ -1157,7 +1229,7 @@ class ShareGroupSnapshot(BASE, ManilaBase):
     status = Column(String(255))
     share_group = orm.relationship(
         ShareGroup,
-        backref="snapshots",
+        backref=orm.backref("snapshots", lazy='joined'),
         foreign_keys=share_group_id,
         primaryjoin=('and_('
                      'ShareGroupSnapshot.share_group_id == ShareGroup.id,'
