@@ -30,7 +30,6 @@ import six
 import webob.exc
 
 from manila.i18n import _
-from manila.i18n import _LE
 
 LOG = log.getLogger(__name__)
 
@@ -92,9 +91,9 @@ class ManilaException(Exception):
             except Exception:
                 # kwargs doesn't match a variable in the message
                 # log the issue and the kwargs
-                LOG.exception(_LE('Exception in string format operation.'))
+                LOG.exception('Exception in string format operation.')
                 for name, value in kwargs.items():
-                    LOG.error(_LE("%(name)s: %(value)s"), {
+                    LOG.error("%(name)s: %(value)s", {
                         'name': name, 'value': value})
                 if CONF.fatal_exception_format_errors:
                     raise
@@ -104,7 +103,7 @@ class ManilaException(Exception):
         elif isinstance(message, Exception):
             message = six.text_type(message)
 
-        if re.match('.*[^\.]\.\.$', message):
+        if re.match(r'.*[^\.]\.\.$', message):
             message = message[:-1]
         self.msg = message
         super(ManilaException, self).__init__(message)
@@ -207,6 +206,10 @@ class NotFound(ManilaException):
     safe = True
 
 
+class MessageNotFound(NotFound):
+    message = _("Message %(message_id)s could not be found.")
+
+
 class Found(ManilaException):
     message = _("Resource was found.")
     code = 302
@@ -225,6 +228,11 @@ class ShareNetworkNotFound(NotFound):
     message = _("Share network %(share_network_id)s could not be found.")
 
 
+class ShareNetworkSubnetNotFound(NotFound):
+    message = _("Share network subnet %(share_network_subnet_id)s could not be"
+                " found.")
+
+
 class ShareServerNotFound(NotFound):
     message = _("Share server %(share_server_id)s could not be found.")
 
@@ -238,8 +246,16 @@ class ShareServerInUse(InUse):
     message = _("Share server %(share_server_id)s is in use.")
 
 
+class ShareServerMigrationError(ManilaException):
+    message = _("Error in share server migration: %(reason)s")
+
+
+class ShareServerMigrationFailed(ManilaException):
+    message = _("Share server migration failed: %(reason)s")
+
+
 class InvalidShareServer(Invalid):
-    message = _("Share server %(share_server_id)s is not valid.")
+    message = _("Invalid share server: %(reason)s")
 
 
 class ShareMigrationError(ManilaException):
@@ -327,6 +343,11 @@ class ProjectUserQuotaNotFound(QuotaNotFound):
                 "could not be found.")
 
 
+class ProjectShareTypeQuotaNotFound(QuotaNotFound):
+    message = _("Quota for share_type %(share_type)s in "
+                "project %(project_id)s could not be found.")
+
+
 class ProjectQuotaNotFound(QuotaNotFound):
     message = _("Quota for project %(project_id)s could not be found.")
 
@@ -387,24 +408,32 @@ class WillNotSchedule(ManilaException):
 class QuotaError(ManilaException):
     message = _("Quota exceeded: code=%(code)s.")
     code = 413
-    headers = {'Retry-After': 0}
+    headers = {'Retry-After': '0'}
     safe = True
 
 
 class ShareSizeExceedsAvailableQuota(QuotaError):
-    message = _("Requested share exceeds allowed gigabytes quota.")
+    message = _(
+        "Requested share exceeds allowed project/user or share type "
+        "gigabytes quota.")
 
 
 class SnapshotSizeExceedsAvailableQuota(QuotaError):
-    message = _("Requested snapshot exceeds allowed gigabytes quota.")
+    message = _(
+        "Requested snapshot exceeds allowed project/user or share type "
+        "gigabytes quota.")
 
 
 class ShareLimitExceeded(QuotaError):
-    message = _("Maximum number of shares allowed (%(allowed)d) exceeded.")
+    message = _(
+        "Maximum number of shares allowed (%(allowed)d) either per "
+        "project/user or share type quota is exceeded.")
 
 
 class SnapshotLimitExceeded(QuotaError):
-    message = _("Maximum number of snapshots allowed (%(allowed)d) exceeded.")
+    message = _(
+        "Maximum number of snapshots allowed (%(allowed)d) either per "
+        "project/user or share type quota is exceeded.")
 
 
 class ShareNetworksLimitExceeded(QuotaError):
@@ -412,6 +441,27 @@ class ShareNetworksLimitExceeded(QuotaError):
                 "allowed (%(allowed)d) exceeded.")
 
 
+class ShareGroupsLimitExceeded(QuotaError):
+    message = _(
+        "Maximum number of allowed share-groups is exceeded.")
+
+
+class ShareGroupSnapshotsLimitExceeded(QuotaError):
+    message = _(
+        "Maximum number of allowed share-group-snapshots is exceeded.")
+
+
+class ShareReplicasLimitExceeded(QuotaError):
+    message = _(
+        "Maximum number of allowed share-replicas is exceeded.")
+
+
+class ShareReplicaSizeExceedsAvailableQuota(QuotaError):
+    message = _(
+        "Requested share replica exceeds allowed project/user or share type "
+        "gigabytes quota.")
+
+
 class GlusterfsException(ManilaException):
     message = _("Unknown Gluster exception.")
 
@@ -433,6 +483,10 @@ class ManageInvalidShare(InvalidShare):
                 "invalid share: %(reason)s")
 
 
+class ManageShareServerError(ManilaException):
+    message = _("Manage existing share server failed due to: %(reason)s")
+
+
 class UnmanageInvalidShare(InvalidShare):
     message = _("Unmanage existing share failed due to "
                 "invalid share: %(reason)s")
@@ -446,6 +500,18 @@ class ShareAccessExists(ManilaException):
     message = _("Share access %(access_type)s:%(access)s exists.")
 
 
+class ShareAccessMetadataNotFound(NotFound):
+    message = _("Share access rule metadata does not exist.")
+
+
+class ShareSnapshotAccessExists(InvalidInput):
+    message = _("Share snapshot access %(access_type)s:%(access)s exists.")
+
+
+class InvalidSnapshotAccess(Invalid):
+    message = _("Invalid access rule: %(reason)s")
+
+
 class InvalidShareAccess(Invalid):
     message = _("Invalid access rule: %(reason)s")
 
@@ -478,6 +544,10 @@ class ShareSnapshotNotSupported(ManilaException):
     message = _("Share %(share_name)s does not support snapshots.")
 
 
+class ShareGroupSnapshotNotSupported(ManilaException):
+    message = _("Share group %(share_group)s does not support snapshots.")
+
+
 class ShareSnapshotIsBusy(ManilaException):
     message = _("Deleting snapshot %(snapshot_name)s that has "
                 "dependent shares.")
@@ -487,6 +557,10 @@ class InvalidShareSnapshot(Invalid):
     message = _("Invalid share snapshot: %(reason)s.")
 
 
+class InvalidShareSnapshotInstance(Invalid):
+    message = _("Invalid share snapshot instance: %(reason)s.")
+
+
 class ManageInvalidShareSnapshot(InvalidShareSnapshot):
     message = _("Manage existing share snapshot failed due to "
                 "invalid share snapshot: %(reason)s.")
@@ -501,11 +575,11 @@ class ShareMetadataNotFound(NotFound):
     message = _("Metadata item is not found.")
 
 
-class InvalidShareMetadata(Invalid):
+class InvalidMetadata(Invalid):
     message = _("Invalid metadata.")
 
 
-class InvalidShareMetadataSize(Invalid):
+class InvalidMetadataSize(Invalid):
     message = _("Invalid metadata size.")
 
 
@@ -531,6 +605,10 @@ class InvalidShareType(Invalid):
     message = _("Invalid share type: %(reason)s.")
 
 
+class InvalidShareGroupType(Invalid):
+    message = _("Invalid share group type: %(reason)s.")
+
+
 class InvalidExtraSpec(Invalid):
     message = _("Invalid extra_spec: %(reason)s.")
 
@@ -547,44 +625,95 @@ class ShareTypeNotFound(NotFound):
     message = _("Share type %(share_type_id)s could not be found.")
 
 
+class ShareGroupTypeNotFound(NotFound):
+    message = _("Share group type %(type_id)s could not be found.")
+
+
 class ShareTypeAccessNotFound(NotFound):
     message = _("Share type access not found for %(share_type_id)s / "
                 "%(project_id)s combination.")
 
 
+class ShareGroupTypeAccessNotFound(NotFound):
+    message = _("Share group type access not found for %(type_id)s / "
+                "%(project_id)s combination.")
+
+
 class ShareTypeNotFoundByName(ShareTypeNotFound):
     message = _("Share type with name %(share_type_name)s "
                 "could not be found.")
 
 
+class ShareGroupTypeNotFoundByName(ShareTypeNotFound):
+    message = _("Share group type with name %(type_name)s "
+                "could not be found.")
+
+
 class ShareTypeExtraSpecsNotFound(NotFound):
     message = _("Share Type %(share_type_id)s has no extra specs with "
                 "key %(extra_specs_key)s.")
 
 
+class ShareGroupTypeSpecsNotFound(NotFound):
+    message = _("Share group type %(type_id)s has no group specs with "
+                "key %(specs_key)s.")
+
+
 class ShareTypeInUse(ManilaException):
-    message = _("Share Type %(share_type_id)s deletion is not allowed with "
-                "shares present with the type.")
+    message = _("Share Type %(share_type_id)s deletion is not allowed while "
+                "shares or share group types are associated with the type.")
 
 
 class IPAddressInUse(InUse):
     message = _("IP address %(ip)s is already used.")
 
 
+class ShareGroupTypeInUse(ManilaException):
+    message = _("Share group Type %(type_id)s deletion is not allowed "
+                "with groups present with the type.")
+
+
 class ShareTypeExists(ManilaException):
     message = _("Share Type %(id)s already exists.")
 
 
+class ShareTypeDoesNotExist(NotFound):
+    message = _("Share Type %(share_type)s does not exist.")
+
+
+class DefaultShareTypeNotConfigured(NotFound):
+    message = _("No default share type is configured. Either configure a "
+                "default share type or explicitly specify a share type.")
+
+
+class ShareGroupTypeExists(ManilaException):
+    message = _("Share group type %(type_id)s already exists.")
+
+
 class ShareTypeAccessExists(ManilaException):
     message = _("Share type access for %(share_type_id)s / "
                 "%(project_id)s combination already exists.")
 
 
+class ShareGroupTypeAccessExists(ManilaException):
+    message = _("Share group type access for %(type_id)s / "
+                "%(project_id)s combination already exists.")
+
+
 class ShareTypeCreateFailed(ManilaException):
     message = _("Cannot create share_type with "
                 "name %(name)s and specs %(extra_specs)s.")
 
 
+class ShareTypeUpdateFailed(ManilaException):
+    message = _("Cannot update share_type %(id)s.")
+
+
+class ShareGroupTypeCreateFailed(ManilaException):
+    message = _("Cannot create share group type with "
+                "name %(name)s and specs %(group_specs)s.")
+
+
 class ManageExistingShareTypeMismatch(ManilaException):
     message = _("Manage existing share failed due to share type mismatch: "
                 "%(reason)s")
@@ -650,6 +779,22 @@ class VserverNotSpecified(NetAppException):
     message = _("Vserver not specified.")
 
 
+class VserverNotReady(NetAppException):
+    message = _("Vserver %(vserver)s is not ready yet.")
+
+
+class EMCPowerMaxXMLAPIError(Invalid):
+    message = _("%(err)s")
+
+
+class EMCPowerMaxLockRequiredException(ManilaException):
+    message = _("Unable to acquire lock(s).")
+
+
+class EMCPowerMaxInvalidMoverID(ManilaException):
+    message = _("Invalid mover or vdm %(id)s.")
+
+
 class EMCVnxXMLAPIError(Invalid):
     message = _("%(err)s")
 
@@ -707,6 +852,10 @@ class HDFSException(ManilaException):
     message = _("HDFS exception occurred!")
 
 
+class MapRFSException(ManilaException):
+    message = _("MapRFS exception occurred: %(msg)s")
+
+
 class ZFSonLinuxException(ManilaException):
     message = _("ZFSonLinux exception occurred: %(msg)s")
 
@@ -734,6 +883,18 @@ class HNASConnException(ManilaException):
     message = _("HNAS Connection Exception: %(msg)s")
 
 
+class HNASSSCIsBusy(ManilaException):
+    message = _("HNAS SSC is busy and cannot execute the command: %(msg)s")
+
+
+class HNASSSCContextChange(ManilaException):
+    message = _("HNAS SSC Context has been changed unexpectedly: %(msg)s")
+
+
+class HNASDirectoryNotEmpty(ManilaException):
+    message = _("HNAS Directory is not empty: %(msg)s")
+
+
 class HNASItemNotFoundException(StorageResourceNotFound):
     message = _("HNAS Item Not Found Exception: %(msg)s")
 
@@ -742,27 +903,27 @@ class HNASNothingToCloneException(ManilaException):
     message = _("HNAS Nothing To Clone Exception: %(msg)s")
 
 
-# ConsistencyGroup
-class ConsistencyGroupNotFound(NotFound):
-    message = _("ConsistencyGroup %(consistency_group_id)s could not be "
+# ShareGroup
+class ShareGroupNotFound(NotFound):
+    message = _("Share group %(share_group_id)s could not be found.")
+
+
+class ShareGroupSnapshotNotFound(NotFound):
+    message = _(
+        "Share group snapshot %(share_group_snapshot_id)s could not be found.")
+
+
+class ShareGroupSnapshotMemberNotFound(NotFound):
+    message = _("Share group snapshot member %(member_id)s could not be "
                 "found.")
 
 
-class CGSnapshotNotFound(NotFound):
-    message = _("Consistency group snapshot %(cgsnapshot_id)s could not be "
-                "found.")
+class InvalidShareGroup(Invalid):
+    message = _("Invalid share group: %(reason)s")
 
 
-class CGSnapshotMemberNotFound(NotFound):
-    message = _("CG snapshot %(member_id)s could not be found.")
-
-
-class InvalidConsistencyGroup(Invalid):
-    message = _("Invalid ConsistencyGroup: %(reason)s")
-
-
-class InvalidCGSnapshot(Invalid):
-    message = _("Invalid CGSnapshot: %(reason)s")
+class InvalidShareGroupSnapshot(Invalid):
+    message = _("Invalid share group snapshot: %(reason)s")
 
 
 class DriverNotInitialized(ManilaException):
@@ -824,3 +985,27 @@ class HSPItemNotFoundException(ShareBackendException):
 
 class NexentaException(ShareBackendException):
     message = _("Exception due to Nexenta failure. %(reason)s")
+
+
+# Tooz locking
+class LockCreationFailed(ManilaException):
+    message = _('Unable to create lock. Coordination backend not started.')
+
+
+class LockingFailed(ManilaException):
+    message = _('Lock acquisition failed.')
+
+
+# Ganesha library
+class GaneshaException(ManilaException):
+    message = _("Unknown NFS-Ganesha library exception.")
+
+
+# Infortrend Storage driver
+class InfortrendCLIException(ShareBackendException):
+    message = _("Infortrend CLI exception: %(err)s "
+                "Return Code: %(rc)s, Output: %(out)s")
+
+
+class InfortrendNASException(ShareBackendException):
+    message = _("Infortrend NAS exception: %(err)s")
