@@ -45,6 +45,12 @@ quota_opts = [
     cfg.IntOpt('quota_share_networks',
                default=10,
                help='Number of share-networks allowed per project.'),
+    cfg.IntOpt('quota_share_replicas',
+               default=100,
+               help='Number of share-replicas allowed per project.'),
+    cfg.IntOpt('quota_replica_gigabytes',
+               default=1000,
+               help='Number of replica gigabytes allowed per project.'),
 
     cfg.IntOpt('quota_share_groups',
                default=50,
@@ -346,9 +352,9 @@ class DbQuotaDriver(object):
 
         # Filter resources
         if has_sync:
-            sync_filt = lambda x: hasattr(x, 'sync')
+            sync_filt = lambda x: hasattr(x, 'sync')  # noqa: E731
         else:
-            sync_filt = lambda x: not hasattr(x, 'sync')
+            sync_filt = lambda x: not hasattr(x, 'sync')  # noqa: E731
         desired = set(keys)
         sub_resources = {k: v for k, v in resources.items()
                          if k in desired and sync_filt(v)}
@@ -378,7 +384,8 @@ class DbQuotaDriver(object):
         return {k: v['limit'] for k, v in quotas.items()}
 
     def reserve(self, context, resources, deltas, expire=None,
-                project_id=None, user_id=None, share_type_id=None):
+                project_id=None, user_id=None, share_type_id=None,
+                overquota_allowed=False):
         """Check quotas and reserve resources.
 
         For counting quotas--those quotas for which there is a usage
@@ -459,7 +466,7 @@ class DbQuotaDriver(object):
             context, resources, quotas, user_quotas, share_type_quotas,
             deltas, expire, CONF.until_refresh, CONF.max_age,
             project_id=project_id, user_id=user_id,
-            share_type_id=share_type_id)
+            share_type_id=share_type_id, overquota_allowed=overquota_allowed)
 
     def commit(self, context, reservations, project_id=None, user_id=None,
                share_type_id=None):
@@ -870,7 +877,7 @@ class QuotaEngine(object):
         return res.count(context, *args, **kwargs)
 
     def reserve(self, context, expire=None, project_id=None, user_id=None,
-                share_type_id=None, **deltas):
+                share_type_id=None, overquota_allowed=False, **deltas):
         """Check quotas and reserve resources.
 
         For counting quotas--those quotas for which there is a usage
@@ -911,6 +918,7 @@ class QuotaEngine(object):
             project_id=project_id,
             user_id=user_id,
             share_type_id=share_type_id,
+            overquota_allowed=overquota_allowed
         )
 
         LOG.debug("Created reservations %s", reservations)
@@ -1059,6 +1067,10 @@ resources = [
                        'quota_share_groups'),
     ReservableResource('share_group_snapshots', '_sync_share_group_snapshots',
                        'quota_share_group_snapshots'),
+    ReservableResource('share_replicas', '_sync_share_replicas',
+                       'quota_share_replicas'),
+    ReservableResource('replica_gigabytes', '_sync_replica_gigabytes',
+                       'quota_replica_gigabytes'),
 ]
 
 
