@@ -36,7 +36,7 @@ from manila.share import share_types
 try:
     import ceph_volume_client
     ceph_module_found = True
-except ImportError as e:
+except ImportError:
     ceph_volume_client = None
     ceph_module_found = False
 
@@ -68,7 +68,11 @@ cephfs_opts = [
                help="The prefix of the cephfs volume path."
                ),
     cfg.BoolOpt('cephfs_enable_snapshots',
-                default=False,
+                deprecated_for_removal=True,
+                deprecated_since='Victoria',
+                deprecated_reason='CephFS snapshots are fully supported '
+                                  'since the Nautilus release of Ceph.',
+                default=True,
                 help="Whether to enable snapshots in this driver."
                 ),
     cfg.StrOpt('cephfs_protocol_helper_type',
@@ -163,8 +167,8 @@ class CephFSDriver(driver.ExecuteMixin, driver.GaneshaMixin,
     def _update_share_stats(self):
         stats = self.volume_client.rados.get_cluster_stats()
 
-        total_capacity_gb = stats['kb'] * units.Mi
-        free_capacity_gb = stats['kb_avail'] * units.Mi
+        total_capacity_gb = round(stats['kb'] / units.Mi, 2)
+        free_capacity_gb = round(stats['kb_avail'] / units.Mi, 2)
 
         data = {
             'vendor_name': 'Ceph',
@@ -178,7 +182,8 @@ class CephFSDriver(driver.ExecuteMixin, driver.GaneshaMixin,
                     'total_capacity_gb': total_capacity_gb,
                     'free_capacity_gb': free_capacity_gb,
                     'qos': 'False',
-                    'reserved_percentage': 0,
+                    'reserved_percentage': self.configuration.safe_get(
+                        'reserved_share_percentage'),
                     'dedupe': [False],
                     'compression': [False],
                     'thin_provisioning': [False]
