@@ -82,13 +82,13 @@ def _conf2json(conf):
 
         for pat, s in [
                 # add omitted "=" signs to block openings
-                ('([^=\s])\s*{', '\\1={'),
+                (r'([^=\s])\s*{', '\\1={'),
                 # delete trailing semicolons in blocks
-                (';\s*}', '}'),
+                (r';\s*}', '}'),
                 # add omitted semicolons after blocks
-                ('}\s*([^}\s])', '};\\1'),
+                (r'}\s*([^}\s])', '};\\1'),
                 # separate syntactically significant characters
-                ('([;{}=])', ' \\1 ')]:
+                (r'([;{}=])', ' \\1 ')]:
             tok = re.sub(pat, s, tok)
 
         # map tokens to JSON equivalents
@@ -98,7 +98,7 @@ def _conf2json(conf):
             elif word == ";":
                 word = ','
             elif (word in ['{', '}'] or
-                  re.search('\A-?[1-9]\d*(\.\d+)?\Z', word)):
+                  re.search(r'\A-?[1-9]\d*(\.\d+)?\Z', word)):
                 pass
             else:
                 word = jsonutils.dumps(word)
@@ -165,23 +165,23 @@ def parseconf(conf):
 
     Convert config to a (nested) dictionary.
     """
-    def list_to_dict(l):
+    def list_to_dict(src_list):
         # Convert a list of key-value pairs stored as tuples to a dict.
         # For tuples with identical keys, preserve all the values in a
         # list. e.g., argument [('k', 'v1'), ('k', 'v2')] to function
         # returns {'k': ['v1', 'v2']}.
-        d = {}
-        for i in l:
+        dst_dict = {}
+        for i in src_list:
             if isinstance(i, tuple):
                 k, v = i
                 if isinstance(v, list):
                     v = list_to_dict(v)
-                if k in d:
-                    d[k] = [d[k]]
-                    d[k].append(v)
+                if k in dst_dict:
+                    dst_dict[k] = [dst_dict[k]]
+                    dst_dict[k].append(v)
                 else:
-                    d[k] = v
-        return d
+                    dst_dict[k] = v
+        return dst_dict
 
     try:
         # allow config to be specified in JSON --
@@ -193,8 +193,8 @@ def parseconf(conf):
         # occurrences of a config block to be later converted to a
         # dict key-value pair, with block name being the key and a
         # list of block contents being the value.
-        l = jsonutils.loads(_conf2json(conf), object_pairs_hook=lambda x: x)
-        d = list_to_dict(l)
+        li = jsonutils.loads(_conf2json(conf), object_pairs_hook=lambda x: x)
+        d = list_to_dict(li)
     return d
 
 
@@ -222,7 +222,7 @@ class GaneshaManager(object):
     """Ganesha instrumentation class."""
 
     def __init__(self, execute, tag, **kwargs):
-        self.confrx = re.compile('\.conf\Z')
+        self.confrx = re.compile(r'\.conf\Z')
         self.ganesha_config_path = kwargs['ganesha_config_path']
         self.tag = tag
 
@@ -481,7 +481,7 @@ class GaneshaManager(object):
             else:
                 _mkindex_called = True
                 self._mkindex()
-        except Exception as e:
+        except exception.ProcessExecutionError as e:
             for u in undos:
                 u()
             if not self.ganesha_rados_store_enable and not _mkindex_called:
@@ -499,7 +499,7 @@ class GaneshaManager(object):
         try:
             self._dbus_send_ganesha("UpdateExport", "string:" + path,
                                     "string:EXPORT(Export_Id=%d)" % xid)
-        except Exception as e:
+        except exception.ProcessExecutionError as e:
             # Revert the export update.
             self._write_export(name, old_confdict)
             raise exception.GaneshaCommandFailure(
@@ -566,7 +566,7 @@ class GaneshaManager(object):
                 "sqlite3", self.ganesha_db_path,
                 bumpcode + 'select * from ganesha where key = "exportid";',
                 run_as_root=False)[0]
-            match = re.search('\Aexportid\|(\d+)$', out)
+            match = re.search(r'\Aexportid\|(\d+)$', out)
             if not match:
                 LOG.error("Invalid export database on "
                           "Ganesha node %(tag)s: %(db)s.",
