@@ -181,6 +181,14 @@ def nfs_synchronized(f):
     return wrapped_func
 
 
+def escaped_address(address):
+    addr = ipaddress.ip_address(six.text_type(address))
+    if addr.version == 4:
+        return six.text_type(addr)
+    else:
+        return '[%s]' % six.text_type(addr)
+
+
 class NFSHelper(NASHelperBase):
     """Interface to work with share."""
 
@@ -191,24 +199,16 @@ class NFSHelper(NASHelperBase):
         if 'public_addresses' in server_copy:
             for address in server_copy['public_addresses']:
                 public_addresses.append(
-                    self._escaped_address(address))
+                    escaped_address(address))
             server_copy['public_addresses'] = public_addresses
 
         for t in ['public_address', 'admin_ip', 'ip']:
             address = server_copy.get(t)
             if address is not None:
-                server_copy[t] = self._escaped_address(address)
+                server_copy[t] = escaped_address(address)
 
         return self.get_exports_for_share(server_copy, path)
 
-    @staticmethod
-    def _escaped_address(address):
-        addr = ipaddress.ip_address(six.text_type(address))
-        if addr.version == 4:
-            return six.text_type(addr)
-        else:
-            return '[%s]' % six.text_type(addr)
-
     def init_helper(self, server):
         try:
             self._ssh_exec(server, ['sudo', 'exportfs'])
@@ -278,15 +278,25 @@ class NFSHelper(NASHelperBase):
                     continue
                 access_to = self._get_parsed_address_or_cidr(
                     access['access_to'])
-                self._ssh_exec(server, ['sudo', 'exportfs', '-u',
-                               ':'.join((access_to, local_path))])
+                try:
+                    self._ssh_exec(server, ['sudo', 'exportfs', '-u',
+                                            ':'.join((access_to, local_path))])
+                except exception.ProcessExecutionError as e:
+                    if "could not find" in e.stderr.lower():
+                        LOG.debug(
+                            "Client/s with IP address/es %(host)s did not "
+                            "have access to %(share)s. Nothing to deny.",
+                            {'host': access_to, 'share': share_name})
+                    else:
+                        raise
+
             if delete_rules:
                 self._sync_nfs_temp_and_perm_files(server)
             for access in add_rules:
                 access_to = self._get_parsed_address_or_cidr(
                     access['access_to'])
                 found_item = re.search(
-                    re.escape(local_path) + '[\s\n]*' + re.escape(access_to),
+                    re.escape(local_path) + r'[\s\n]*' + re.escape(access_to),
                     out)
                 if found_item is not None:
                     LOG.warning("Access rule %(type)s:%(to)s already "
@@ -344,11 +354,12 @@ class NFSHelper(NASHelperBase):
         self._ssh_exec(server, sync_cmd)
         self._ssh_exec(server, ['sudo', 'exportfs', '-a'])
         out, _ = self._ssh_exec(
-            server, ['sudo', 'service', 'nfs-kernel-server', 'status'],
+            server,
+            ['sudo', 'systemctl', 'is-active', 'nfs-kernel-server'],
             check_exit_code=False)
-        if "not" in out:
+        if "inactive" in out:
             self._ssh_exec(
-                server, ['sudo', 'service', 'nfs-kernel-server', 'restart'])
+                server, ['sudo', 'systemctl', 'restart', 'nfs-kernel-server'])
 
     def _get_export_location_template(self, export_location_or_path):
         path = export_location_or_path.split(':')[-1]
@@ -447,7 +458,7 @@ class CIFSHelperIPAccess(CIFSHelperBase):
             except Exception:
                 msg = _("Could not create CIFS export %s.") % share_name
                 LOG.exception(msg)
-                raise exception.ManilaException(reason=msg)
+                raise exception.ManilaException(msg)
         else:
             # Share exists
             if recreate:
@@ -458,7 +469,7 @@ class CIFSHelperIPAccess(CIFSHelperBase):
                 except Exception:
                     msg = _("Could not create CIFS export %s.") % share_name
                     LOG.exception(msg)
-                    raise exception.ManilaException(reason=msg)
+                    raise exception.ManilaException(msg)
             else:
                 msg = _('Share section %s already defined.') % share_name
                 raise exception.ShareBackendException(msg=msg)
@@ -529,6 +540,30 @@ class CIFSHelperIPAccess(CIFSHelperBase):
         ]
         self._ssh_exec(server, backup_exports)
         self._set_allow_hosts(server, [], share_name)
+        self._kick_out_users(server, share_name)
+
+    def _kick_out_users(self, server, share_name):
+        """Kick out all users of share"""
+        (out, _) = self._ssh_exec(server, ['sudo', 'smbstatus', '-S'])
+
+        shares = []
+        header = True
+        regexp = r"^(?P<share>[^ ]+)\s+(?P<pid>[0-9]+)\s+(?P<machine>[^ ]+).*"
+        for line in out.splitlines():
+            line = line.strip()
+            if not header and line:
+                match = re.match(regexp, line)
+                if match:
+                    shares.append(match.groupdict())
+                else:
+                    raise exception.ShareBackendException(
+                        msg="Failed to obtain smbstatus for %s!" % share_name)
+            elif line.startswith('----'):
+                header = False
+        to_kill = [s['pid'] for s in shares if
+                   share_name == s['share'] or share_name is None]
+        if to_kill:
+            self._ssh_exec(server, ['sudo', 'kill', '-15'] + to_kill)
 
     def restore_access_after_maintenance(self, server, share_name):
         maintenance_file = self._get_maintenance_file_path(share_name)
