@@ -21,8 +21,9 @@ from oslo_log import log
 
 from manila.common import constants
 from manila import exception
-from manila.i18n import _, _LW
+from manila.i18n import _
 from manila.share import api as share_api
+from manila.share import rpcapi as share_rpcapi
 import manila.utils as utils
 
 
@@ -47,12 +48,13 @@ CONF.register_opts(migration_opts)
 
 class ShareMigrationHelper(object):
 
-    def __init__(self, context, db, share):
+    def __init__(self, context, db, access_helper):
 
         self.db = db
-        self.share = share
         self.context = context
+        self.access_helper = access_helper
         self.api = share_api.API()
+        self.access_helper = access_helper
 
         self.migration_create_delete_share_timeout = (
             CONF.migration_create_delete_share_timeout)
@@ -81,7 +83,8 @@ class ShareMigrationHelper(object):
             except exception.NotFound:
                 instance = None
             else:
-                time.sleep(tries ** 2)
+                # 1.414 = square-root of 2
+                time.sleep(1.414 ** tries)
 
     def create_instance_and_wait(self, share, dest_host, new_share_network_id,
                                  new_az_id, new_share_type_id):
@@ -114,7 +117,8 @@ class ShareMigrationHelper(object):
                 self.cleanup_new_instance(new_share_instance)
                 raise exception.ShareMigrationFailed(reason=msg)
             else:
-                time.sleep(tries ** 2)
+                # 1.414 = square-root of 2
+                time.sleep(1.414 ** tries)
             new_share_instance = self.db.share_instance_get(
                 self.context, new_share_instance['id'], with_share_data=True)
 
@@ -128,81 +132,59 @@ class ShareMigrationHelper(object):
         try:
             self.delete_instance_and_wait(new_instance)
         except Exception:
-            LOG.warning(_LW("Failed to cleanup new instance during generic"
-                        " migration for share %s."), self.share['id'])
+            LOG.warning("Failed to cleanup new instance during generic "
+                        "migration for share %s.", new_instance['share_id'])
+
+    def cleanup_access_rules(self, share_instances, share_server,
+                             dest_host=None):
 
-    def cleanup_access_rules(self, share_instance, share_server, driver):
         try:
-            self.revert_access_rules(share_instance, share_server, driver)
+            self.revert_access_rules(share_instances, share_server, dest_host)
         except Exception:
-            LOG.warning(_LW("Failed to cleanup access rules during generic"
-                        " migration for share %s."), self.share['id'])
+            LOG.warning("Failed to cleanup access rules during generic"
+                        " migration.")
 
-    def change_to_read_only(self, share_instance, share_server,
-                            readonly_support, driver):
+    def revert_access_rules(self, share_instances, share_server,
+                            dest_host=None):
+        shares_instance_ids = []
+        for share_instance in share_instances:
+            # Cast all rules to 'queued_to_apply' so that they can be
+            # re-applied.
+            shares_instance_ids.append(share_instance['id'])
+            updates = {'state': constants.ACCESS_STATE_QUEUED_TO_APPLY}
+            self.access_helper.get_and_update_share_instance_access_rules(
+                self.context, updates=updates,
+                share_instance_id=share_instance['id'])
 
-        # NOTE(ganso): If the share does not allow readonly mode we
-        # should remove all access rules and prevent any access
+        if dest_host:
+            rpcapi = share_rpcapi.ShareAPI()
+            rpcapi.update_access_for_instances(self.context, dest_host,
+                                               shares_instance_ids,
+                                               share_server)
+        else:
+            for share_instance in share_instances:
+                self.access_helper.update_access_rules(
+                    self.context, share_instance['id'],
+                    share_server=share_server)
 
-        rules = self.db.share_access_get_all_for_instance(
-            self.context, share_instance['id'])
+        for share_instance in share_instances:
+            utils.wait_for_access_update(
+                self.context, self.db, share_instance,
+                self.migration_wait_access_rules_timeout)
 
-        if len(rules) > 0:
+    def apply_new_access_rules(self, new_share_instance, share_id):
 
-            if readonly_support:
+        rules = self.db.share_instance_access_copy(
+            self.context, share_id, new_share_instance['id'])
 
-                LOG.debug("Changing all of share %s access rules "
-                          "to read-only.", self.share['id'])
+        if rules:
+            self.api.allow_access_to_instance(self.context, new_share_instance)
 
-                for rule in rules:
-                    rule['access_level'] = constants.ACCESS_LEVEL_RO
-
-                driver.update_access(self.context, share_instance, rules,
-                                     add_rules=[], delete_rules=[],
-                                     share_server=share_server)
-            else:
-
-                LOG.debug("Removing all access rules for migration of "
-                          "share %s." % self.share['id'])
-
-                driver.update_access(self.context, share_instance, [],
-                                     add_rules=[], delete_rules=rules,
-                                     share_server=share_server)
-
-    def revert_access_rules(self, share_instance, share_server, driver):
-
-        rules = self.db.share_access_get_all_for_instance(
-            self.context, share_instance['id'])
-
-        if len(rules) > 0:
-            LOG.debug("Restoring all of share %s access rules according to "
-                      "DB.", self.share['id'])
-
-            driver.update_access(self.context, share_instance, rules,
-                                 add_rules=[], delete_rules=[],
-                                 share_server=share_server)
-
-    def apply_new_access_rules(self, new_share_instance):
-
-        self.db.share_instance_access_copy(self.context, self.share['id'],
-                                           new_share_instance['id'])
-
-        rules = self.db.share_access_get_all_for_instance(
-            self.context, new_share_instance['id'])
-
-        if len(rules) > 0:
-            LOG.debug("Restoring all of share %s access rules according to "
-                      "DB.", self.share['id'])
-
-            # refresh share instance
-            new_share_instance = self.db.share_instance_get(
-                self.context, new_share_instance['id'], with_share_data=True)
-
-            self.api.allow_access_to_instance(self.context, new_share_instance,
-                                              rules)
             utils.wait_for_access_update(
                 self.context, self.db, new_share_instance,
                 self.migration_wait_access_rules_timeout)
+        else:
+            LOG.debug("No access rules to sync to destination share instance.")
 
     @utils.retry(exception.ShareServerNotReady, retries=8)
     def wait_for_share_server(self, share_server_id):
