@@ -15,21 +15,21 @@
 
 import copy
 import datetime
+from unittest import mock
 
 import ddt
-import mock
 from oslo_config import cfg
 from oslo_serialization import jsonutils
 import six
 import webob
 
 from manila.api import common
-from manila.api.openstack import api_version_request as api_version
 from manila.api.v1 import shares
 from manila.common import constants
 from manila import context
 from manila import db
 from manila import exception
+from manila import policy
 from manila.share import api as share_api
 from manila.share import share_types
 from manila import test
@@ -46,7 +46,7 @@ class ShareAPITest(test.TestCase):
     """Share API Test."""
 
     def setUp(self):
-        super(self.__class__, self).setUp()
+        super(ShareAPITest, self).setUp()
         self.controller = shares.ShareController()
         self.mock_object(db, 'availability_zone_get')
         self.mock_object(share_api.API, 'get_all',
@@ -57,6 +57,13 @@ class ShareAPITest(test.TestCase):
         self.mock_object(share_api.API, 'delete', stubs.stub_share_delete)
         self.mock_object(share_api.API, 'get_snapshot',
                          stubs.stub_snapshot_get)
+        self.mock_object(share_types, 'get_share_type',
+                         stubs.stub_share_type_get)
+        self.mock_object(
+            common, 'validate_public_share_policy',
+            mock.Mock(side_effect=lambda *args, **kwargs: args[1]))
+        self.resource_name = self.controller.resource_name
+        self.mock_policy_check = self.mock_object(policy, 'check_policy')
         self.maxDiff = None
         self.share = {
             "size": 100,
@@ -78,7 +85,14 @@ class ShareAPITest(test.TestCase):
         self.vt = {
             'id': 'fake_volume_type_id',
             'name': 'fake_volume_type_name',
+            'required_extra_specs': {
+                'driver_handles_share_servers': 'False'
+            },
+            'extra_specs': {
+                'driver_handles_share_servers': 'False'
+            }
         }
+
         CONF.set_default("default_share_type", None)
 
     def _get_expected_share_detailed_response(self, values=None, admin=False):
@@ -90,7 +104,6 @@ class ShareAPITest(test.TestCase):
             'export_location': 'fake_location',
             'export_locations': ['fake_location', 'fake_location2'],
             'project_id': 'fakeproject',
-            'host': 'fakehost',
             'created_at': datetime.datetime(1, 1, 1, 1, 1, 1),
             'share_proto': 'FAKEPROTO',
             'metadata': {},
@@ -123,6 +136,7 @@ class ShareAPITest(test.TestCase):
             share['share_proto'] = share['share_proto'].upper()
         if admin:
             share['share_server_id'] = 'fake_share_server_id'
+            share['host'] = 'fakehost'
         return {'share': share}
 
     @ddt.data("1.0", "2.0", "2.1")
@@ -133,6 +147,8 @@ class ShareAPITest(test.TestCase):
 
         res_dict = self.controller.create(req, body)
 
+        self.mock_policy_check.assert_called_once_with(
+            req.environ['manila.context'], self.resource_name, 'create')
         expected = self._get_expected_share_detailed_response(self.share)
         expected['share'].pop('snapshot_support')
         self.assertEqual(expected, res_dict)
@@ -145,25 +161,11 @@ class ShareAPITest(test.TestCase):
 
         res_dict = self.controller.create(req, body)
 
+        self.mock_policy_check.assert_called_once_with(
+            req.environ['manila.context'], self.resource_name, 'create')
         expected = self._get_expected_share_detailed_response(self.share)
         self.assertEqual(expected, res_dict)
 
-    @ddt.data("2.4", "2.5")
-    def test_share_create_with_consistency_group(self, microversion):
-        self.mock_object(share_api.API, 'create', self.create_mock)
-        body = {"share": copy.deepcopy(self.share)}
-        req = fakes.HTTPRequest.blank('/shares', version=microversion)
-
-        res_dict = self.controller.create(req, body)
-
-        expected = self._get_expected_share_detailed_response(self.share)
-        expected['share']['consistency_group_id'] = None
-        expected['share']['source_cgsnapshot_member_id'] = None
-        if (api_version.APIVersionRequest(microversion) >=
-                api_version.APIVersionRequest('2.5')):
-            expected['share']['task_state'] = None
-        self.assertEqual(expected, res_dict)
-
     def test_share_create_with_valid_default_share_type(self):
         self.mock_object(share_types, 'get_share_type_by_name',
                          mock.Mock(return_value=self.vt))
@@ -174,6 +176,8 @@ class ShareAPITest(test.TestCase):
         req = fakes.HTTPRequest.blank('/shares')
         res_dict = self.controller.create(req, body)
 
+        self.mock_policy_check.assert_called_once_with(
+            req.environ['manila.context'], self.resource_name, 'create')
         expected = self._get_expected_share_detailed_response(self.share)
         expected['share'].pop('snapshot_support')
         share_types.get_share_type_by_name.assert_called_once_with(
@@ -188,8 +192,11 @@ class ShareAPITest(test.TestCase):
         )
         CONF.set_default("default_share_type", self.vt['name'])
         req = fakes.HTTPRequest.blank('/shares')
+
         self.assertRaises(exception.ShareTypeNotFoundByName,
                           self.controller.create, req, {'share': self.share})
+        self.mock_policy_check.assert_called_once_with(
+            req.environ['manila.context'], self.resource_name, 'create')
         share_types.get_default_share_type.assert_called_once_with()
 
     def test_share_create_with_dhss_true_and_network_notexist(self):
@@ -206,8 +213,11 @@ class ShareAPITest(test.TestCase):
         )
         CONF.set_default("default_share_type", fake_share_type['name'])
         req = fakes.HTTPRequest.blank('/shares')
+
         self.assertRaises(webob.exc.HTTPBadRequest,
                           self.controller.create, req, {'share': self.share})
+        self.mock_policy_check.assert_called_once_with(
+            req.environ['manila.context'], self.resource_name, 'create')
         share_types.get_default_share_type.assert_called_once_with()
 
     def test_share_create_with_share_net(self):
@@ -229,14 +239,20 @@ class ShareAPITest(test.TestCase):
         self.mock_object(share_api.API, 'create', create_mock)
         self.mock_object(share_api.API, 'get_share_network', mock.Mock(
             return_value={'id': 'fakenetid'}))
+        self.mock_object(
+            db, 'share_network_subnet_get_by_availability_zone_id',
+            mock.Mock(return_value={'id': 'fakesubnetid'}))
 
         body = {"share": copy.deepcopy(shr)}
         req = fakes.HTTPRequest.blank('/shares')
         res_dict = self.controller.create(req, body)
 
+        self.mock_policy_check.assert_called_once_with(
+            req.environ['manila.context'], self.resource_name, 'create')
         expected = self._get_expected_share_detailed_response(shr)
         expected['share'].pop('snapshot_support')
         self.assertEqual(expected, res_dict)
+        # pylint: disable=unsubscriptable-object
         self.assertEqual("fakenetid",
                          create_mock.call_args[1]['share_network_id'])
 
@@ -261,7 +277,11 @@ class ShareAPITest(test.TestCase):
         self.mock_object(share_api.API, 'create', create_mock)
         body = {"share": copy.deepcopy(shr)}
         req = fakes.HTTPRequest.blank('/shares')
+
         res_dict = self.controller.create(req, body)
+
+        self.mock_policy_check.assert_called_once_with(
+            req.environ['manila.context'], self.resource_name, 'create')
         expected = self._get_expected_share_detailed_response(shr)
         expected['share'].pop('snapshot_support')
         self.assertEqual(expected, res_dict)
@@ -289,18 +309,27 @@ class ShareAPITest(test.TestCase):
         self.mock_object(share_api.API, 'create', create_mock)
         self.mock_object(share_api.API, 'get_snapshot',
                          stubs.stub_snapshot_get)
+        parent_share = stubs.stub_share(
+            '1', instance={'share_network_id': parent_share_net},
+            create_share_from_snapshot_support=True)
         self.mock_object(share_api.API, 'get', mock.Mock(
-            return_value=mock.Mock(
-                instance={'share_network_id': parent_share_net})))
+            return_value=parent_share))
         self.mock_object(share_api.API, 'get_share_network', mock.Mock(
             return_value={'id': parent_share_net}))
+        self.mock_object(
+            db, 'share_network_subnet_get_by_availability_zone_id')
 
         body = {"share": copy.deepcopy(shr)}
         req = fakes.HTTPRequest.blank('/shares')
+
         res_dict = self.controller.create(req, body)
+
+        self.mock_policy_check.assert_called_once_with(
+            req.environ['manila.context'], self.resource_name, 'create')
         expected = self._get_expected_share_detailed_response(shr)
         expected['share'].pop('snapshot_support')
         self.assertEqual(expected, res_dict)
+        # pylint: disable=unsubscriptable-object
         self.assertEqual(parent_share_net,
                          create_mock.call_args[1]['share_network_id'])
 
@@ -327,18 +356,27 @@ class ShareAPITest(test.TestCase):
         self.mock_object(share_api.API, 'create', create_mock)
         self.mock_object(share_api.API, 'get_snapshot',
                          stubs.stub_snapshot_get)
+        parent_share = stubs.stub_share(
+            '1', instance={'share_network_id': parent_share_net},
+            create_share_from_snapshot_support=True)
         self.mock_object(share_api.API, 'get', mock.Mock(
-            return_value=mock.Mock(
-                instance={'share_network_id': parent_share_net})))
+            return_value=parent_share))
         self.mock_object(share_api.API, 'get_share_network', mock.Mock(
             return_value={'id': parent_share_net}))
+        self.mock_object(
+            db, 'share_network_subnet_get_by_availability_zone_id')
 
         body = {"share": copy.deepcopy(shr)}
         req = fakes.HTTPRequest.blank('/shares')
+
         res_dict = self.controller.create(req, body)
+
+        self.mock_policy_check.assert_called_once_with(
+            req.environ['manila.context'], self.resource_name, 'create')
         expected = self._get_expected_share_detailed_response(shr)
         expected['share'].pop('snapshot_support')
         self.assertEqual(expected, res_dict)
+        # pylint: disable=unsubscriptable-object
         self.assertEqual(parent_share_net,
                          create_mock.call_args[1]['share_network_id'])
 
@@ -355,10 +393,64 @@ class ShareAPITest(test.TestCase):
         }
         body = {"share": shr}
         req = fakes.HTTPRequest.blank('/shares')
+
         self.assertRaises(webob.exc.HTTPBadRequest,
                           self.controller.create,
                           req,
                           body)
+        self.mock_policy_check.assert_called_once_with(
+            req.environ['manila.context'], self.resource_name, 'create')
+
+    @ddt.data("1.0", "2.0")
+    def test_share_create_from_snapshot_not_supported(self, microversion):
+        # This create operation should work, because the 1.0 API doesn't check
+        # create_share_from_snapshot_support.
+
+        parent_share_net = 444
+        shr = {
+            "size": 100,
+            "name": "Share Test Name",
+            "description": "Share Test Desc",
+            "share_proto": "fakeproto",
+            "availability_zone": "zone1:host1",
+            "snapshot_id": 333,
+            "share_network_id": parent_share_net
+        }
+        create_mock = mock.Mock(return_value=stubs.stub_share('1',
+                                display_name=shr['name'],
+                                display_description=shr['description'],
+                                size=shr['size'],
+                                share_proto=shr['share_proto'].upper(),
+                                snapshot_id=shr['snapshot_id'],
+                                instance=dict(
+                                    availability_zone=shr['availability_zone'],
+                                    share_network_id=shr['share_network_id'])))
+        self.mock_object(share_api.API, 'create', create_mock)
+        self.mock_object(share_api.API, 'get_snapshot',
+                         stubs.stub_snapshot_get)
+        parent_share = stubs.stub_share(
+            '1', instance={'share_network_id': parent_share_net},
+            create_share_from_snapshot_support=False)
+        self.mock_object(share_api.API, 'get', mock.Mock(
+            return_value=parent_share))
+        self.mock_object(share_api.API, 'get_share_network', mock.Mock(
+            return_value={'id': parent_share_net}))
+        self.mock_object(
+            db, 'share_network_subnet_get_by_availability_zone_id')
+
+        body = {"share": copy.deepcopy(shr)}
+        req = fakes.HTTPRequest.blank('/shares', version=microversion)
+
+        res_dict = self.controller.create(req, body)
+
+        self.mock_policy_check.assert_called_once_with(
+            req.environ['manila.context'], self.resource_name, 'create')
+        expected = self._get_expected_share_detailed_response(shr)
+        expected['share'].pop('snapshot_support')
+        self.assertDictEqual(expected, res_dict)
+        # pylint: disable=unsubscriptable-object
+        self.assertEqual(parent_share_net,
+                         create_mock.call_args[1]['share_network_id'])
 
     def test_share_creation_fails_with_bad_size(self):
         shr = {"size": '',
@@ -372,6 +464,8 @@ class ShareAPITest(test.TestCase):
                           self.controller.create,
                           req,
                           body)
+        self.mock_policy_check.assert_called_once_with(
+            req.environ['manila.context'], self.resource_name, 'create')
 
     def test_share_create_no_body(self):
         body = {}
@@ -380,6 +474,8 @@ class ShareAPITest(test.TestCase):
                           self.controller.create,
                           req,
                           body)
+        self.mock_policy_check.assert_called_once_with(
+            req.environ['manila.context'], self.resource_name, 'create')
 
     def test_share_create_invalid_availability_zone(self):
         self.mock_object(
@@ -395,6 +491,28 @@ class ShareAPITest(test.TestCase):
                           req,
                           body)
 
+    @ddt.data((exception.ShareNetworkNotFound(share_network_id='fake'),
+               webob.exc.HTTPNotFound),
+              (mock.Mock(), webob.exc.HTTPBadRequest))
+    @ddt.unpack
+    def test_share_create_invalid_subnet(self, share_network_side_effect,
+                                         exception_to_raise):
+        fake_share_with_sn = copy.deepcopy(self.share)
+        fake_share_with_sn['share_network_id'] = 'fakenetid'
+        self.mock_object(db, 'share_network_get',
+                         mock.Mock(side_effect=share_network_side_effect))
+        self.mock_object(
+            db, 'share_network_subnet_get_by_availability_zone_id',
+            mock.Mock(return_value=None))
+
+        body = {"share": fake_share_with_sn}
+
+        req = fakes.HTTPRequest.blank('/shares')
+        self.assertRaises(exception_to_raise,
+                          self.controller.create,
+                          req,
+                          body)
+
     def test_share_show(self):
         req = fakes.HTTPRequest.blank('/shares/1')
         expected = self._get_expected_share_detailed_response()
@@ -404,22 +522,10 @@ class ShareAPITest(test.TestCase):
 
         self.assertEqual(expected, res_dict)
 
-    def test_share_show_with_consistency_group(self):
-        req = fakes.HTTPRequest.blank('/shares/1', version='2.4')
-        expected = self._get_expected_share_detailed_response()
-        expected['share']['consistency_group_id'] = None
-        expected['share']['source_cgsnapshot_member_id'] = None
-
-        res_dict = self.controller.show(req, '1')
-
-        self.assertEqual(expected, res_dict)
-
     def test_share_show_with_share_type_name(self):
         req = fakes.HTTPRequest.blank('/shares/1', version='2.6')
         res_dict = self.controller.show(req, '1')
         expected = self._get_expected_share_detailed_response()
-        expected['share']['consistency_group_id'] = None
-        expected['share']['source_cgsnapshot_member_id'] = None
         expected['share']['share_type_name'] = None
         expected['share']['task_state'] = None
         self.assertEqual(expected, res_dict)
@@ -446,59 +552,29 @@ class ShareAPITest(test.TestCase):
         resp = self.controller.delete(req, 1)
         self.assertEqual(202, resp.status_int)
 
-    def test_share_delete_in_consistency_group_param_not_provided(self):
-        fake_share = stubs.stub_share('fake_share',
-                                      consistency_group_id='fake_cg_id')
-        self.mock_object(share_api.API, 'get',
-                         mock.Mock(return_value=fake_share))
-        req = fakes.HTTPRequest.blank('/shares/1')
-        self.assertRaises(webob.exc.HTTPBadRequest,
-                          self.controller.delete, req, 1)
-
-    def test_share_delete_in_consistency_group(self):
-        fake_share = stubs.stub_share('fake_share',
-                                      consistency_group_id='fake_cg_id')
-        self.mock_object(share_api.API, 'get',
-                         mock.Mock(return_value=fake_share))
-        req = fakes.HTTPRequest.blank(
-            '/shares/1?consistency_group_id=fake_cg_id')
-        resp = self.controller.delete(req, 1)
-        self.assertEqual(202, resp.status_int)
-
-    def test_share_delete_in_consistency_group_wrong_id(self):
-        fake_share = stubs.stub_share('fake_share',
-                                      consistency_group_id='fake_cg_id')
-        self.mock_object(share_api.API, 'get',
-                         mock.Mock(return_value=fake_share))
-        req = fakes.HTTPRequest.blank(
-            '/shares/1?consistency_group_id=not_fake_cg_id')
-        self.assertRaises(webob.exc.HTTPBadRequest,
-                          self.controller.delete, req, 1)
-
     def test_share_update(self):
         shr = self.share
         body = {"share": shr}
 
         req = fakes.HTTPRequest.blank('/share/1')
+
         res_dict = self.controller.update(req, 1, body)
+
+        self.mock_policy_check.assert_called_once_with(
+            req.environ['manila.context'], self.resource_name, 'update')
         self.assertEqual(shr["display_name"], res_dict['share']["name"])
         self.assertEqual(shr["display_description"],
                          res_dict['share']["description"])
         self.assertEqual(shr['is_public'],
                          res_dict['share']['is_public'])
 
-    def test_share_update_with_consistency_group(self):
-        shr = self.share
-        body = {"share": shr}
-
-        req = fakes.HTTPRequest.blank('/share/1', version="2.4")
-        res_dict = self.controller.update(req, 1, body)
-        self.assertIsNone(res_dict['share']["consistency_group_id"])
-        self.assertIsNone(res_dict['share']["source_cgsnapshot_member_id"])
-
     def test_share_not_updates_size(self):
         req = fakes.HTTPRequest.blank('/share/1')
+
         res_dict = self.controller.update(req, 1, {"share": self.share})
+
+        self.mock_policy_check.assert_called_once_with(
+            req.environ['manila.context'], self.resource_name, 'update')
         self.assertNotEqual(res_dict['share']["size"], self.share["size"])
 
     def test_share_delete_no_share(self):
@@ -517,7 +593,6 @@ class ShareAPITest(test.TestCase):
             'share_server_id': 'fake_share_server_id',
             'share_type_id': 'fake_share_type_id',
             'snapshot_id': 'fake_snapshot_id',
-            'host': 'fake_host',
             'share_network_id': 'fake_share_network_id',
             'metadata': '%7B%27k1%27%3A+%27v1%27%7D',  # serialized k1=v1
             'extra_specs': '%7B%27k2%27%3A+%27v2%27%7D',  # serialized k2=v2
@@ -527,6 +602,8 @@ class ShareAPITest(test.TestCase):
             'offset': '1',
             'is_public': 'False',
         }
+        if use_admin_context:
+            search_opts['host'] = 'fake_host'
         # fake_key should be filtered for non-admin
         url = '/shares?fake_key=fake_value'
         for k, v in search_opts.items():
@@ -539,7 +616,7 @@ class ShareAPITest(test.TestCase):
             {'id': 'id3', 'display_name': 'n3'},
         ]
         self.mock_object(share_api.API, 'get_all',
-                         mock.Mock(return_value=shares))
+                         mock.Mock(return_value=[shares[1]]))
 
         result = self.controller.index(req)
 
@@ -549,14 +626,17 @@ class ShareAPITest(test.TestCase):
             'share_server_id': search_opts['share_server_id'],
             'share_type_id': search_opts['share_type_id'],
             'snapshot_id': search_opts['snapshot_id'],
-            'host': search_opts['host'],
             'share_network_id': search_opts['share_network_id'],
             'metadata': {'k1': 'v1'},
             'extra_specs': {'k2': 'v2'},
             'is_public': 'False',
+            'limit': '1',
+            'offset': '1'
         }
+
         if use_admin_context:
             search_opts_expected.update({'fake_key': 'fake_value'})
+            search_opts_expected['host'] = search_opts['host']
         share_api.API.get_all.assert_called_once_with(
             req.environ['manila.context'],
             sort_key=search_opts['sort_key'],
@@ -606,7 +686,6 @@ class ShareAPITest(test.TestCase):
             'share_server_id': 'fake_share_server_id',
             'share_type_id': 'fake_share_type_id',
             'snapshot_id': 'fake_snapshot_id',
-            'host': 'fake_host',
             'share_network_id': 'fake_share_network_id',
             'metadata': '%7B%27k1%27%3A+%27v1%27%7D',  # serialized k1=v1
             'extra_specs': '%7B%27k2%27%3A+%27v2%27%7D',  # serialized k2=v2
@@ -616,6 +695,8 @@ class ShareAPITest(test.TestCase):
             'offset': '1',
             'is_public': 'False',
         }
+        if use_admin_context:
+            search_opts['host'] = 'fake_host'
         # fake_key should be filtered for non-admin
         url = '/shares/detail?fake_key=fake_value'
         for k, v in search_opts.items():
@@ -629,14 +710,14 @@ class ShareAPITest(test.TestCase):
                 'display_name': 'n2',
                 'status': constants.STATUS_AVAILABLE,
                 'snapshot_id': 'fake_snapshot_id',
-                'share_type_id': 'fake_share_type_id',
                 'instance': {'host': 'fake_host',
-                             'share_network_id': 'fake_share_network_id'},
+                             'share_network_id': 'fake_share_network_id',
+                             'share_type_id': 'fake_share_type_id'},
             },
             {'id': 'id3', 'display_name': 'n3'},
         ]
         self.mock_object(share_api.API, 'get_all',
-                         mock.Mock(return_value=shares))
+                         mock.Mock(return_value=[shares[1]]))
 
         result = self.controller.detail(req)
 
@@ -646,14 +727,16 @@ class ShareAPITest(test.TestCase):
             'share_server_id': search_opts['share_server_id'],
             'share_type_id': search_opts['share_type_id'],
             'snapshot_id': search_opts['snapshot_id'],
-            'host': search_opts['host'],
             'share_network_id': search_opts['share_network_id'],
             'metadata': {'k1': 'v1'},
             'extra_specs': {'k2': 'v2'},
             'is_public': 'False',
+            'limit': '1',
+            'offset': '1'
         }
         if use_admin_context:
             search_opts_expected.update({'fake_key': 'fake_value'})
+            search_opts_expected['host'] = search_opts['host']
         share_api.API.get_all.assert_called_once_with(
             req.environ['manila.context'],
             sort_key=search_opts['sort_key'],
@@ -669,11 +752,13 @@ class ShareAPITest(test.TestCase):
         self.assertEqual(
             shares[1]['status'], result['shares'][0]['status'])
         self.assertEqual(
-            shares[1]['share_type_id'], result['shares'][0]['share_type'])
+            shares[1]['instance']['share_type_id'],
+            result['shares'][0]['share_type'])
         self.assertEqual(
             shares[1]['snapshot_id'], result['shares'][0]['snapshot_id'])
-        self.assertEqual(
-            shares[1]['instance']['host'], result['shares'][0]['host'])
+        if use_admin_context:
+            self.assertEqual(
+                shares[1]['instance']['host'], result['shares'][0]['host'])
         self.assertEqual(
             shares[1]['instance']['share_network_id'],
             result['shares'][0]['share_network_id'])
@@ -684,42 +769,41 @@ class ShareAPITest(test.TestCase):
     def test_share_list_detail_with_search_opts_by_admin(self):
         self._share_list_detail_with_search_opts(use_admin_context=True)
 
-    def _list_detail_common_expected(self):
-        return {
-            'shares': [
+    def _list_detail_common_expected(self, admin=False):
+        share_dict = {
+            'status': 'fakestatus',
+            'description': 'displaydesc',
+            'export_location': 'fake_location',
+            'export_locations': ['fake_location', 'fake_location2'],
+            'availability_zone': 'fakeaz',
+            'name': 'displayname',
+            'share_proto': 'FAKEPROTO',
+            'metadata': {},
+            'project_id': 'fakeproject',
+
+            'id': '1',
+            'snapshot_id': '2',
+            'snapshot_support': True,
+            'share_network_id': None,
+            'created_at': datetime.datetime(1, 1, 1, 1, 1, 1),
+            'size': 1,
+            'share_type': '1',
+            'volume_type': '1',
+            'is_public': False,
+            'links': [
                 {
-                    'status': 'fakestatus',
-                    'description': 'displaydesc',
-                    'export_location': 'fake_location',
-                    'export_locations': ['fake_location', 'fake_location2'],
-                    'availability_zone': 'fakeaz',
-                    'name': 'displayname',
-                    'share_proto': 'FAKEPROTO',
-                    'metadata': {},
-                    'project_id': 'fakeproject',
-                    'host': 'fakehost',
-                    'id': '1',
-                    'snapshot_id': '2',
-                    'snapshot_support': True,
-                    'share_network_id': None,
-                    'created_at': datetime.datetime(1, 1, 1, 1, 1, 1),
-                    'size': 1,
-                    'share_type': '1',
-                    'volume_type': '1',
-                    'is_public': False,
-                    'links': [
-                        {
-                            'href': 'http://localhost/v1/fake/shares/1',
-                            'rel': 'self'
-                        },
-                        {
-                            'href': 'http://localhost/fake/shares/1',
-                            'rel': 'bookmark'
-                        }
-                    ],
+                    'href': 'http://localhost/v1/fake/shares/1',
+                    'rel': 'self'
+                },
+                {
+                    'href': 'http://localhost/fake/shares/1',
+                    'rel': 'bookmark'
                 }
-            ]
+            ],
         }
+        if admin:
+            share_dict['host'] = 'fakehost'
+        return {'shares': [share_dict]}
 
     def _list_detail_test_common(self, req, expected):
         self.mock_object(share_api.API, 'get_all',
@@ -736,22 +820,11 @@ class ShareAPITest(test.TestCase):
         expected['shares'][0].pop('snapshot_support')
         self._list_detail_test_common(req, expected)
 
-    def test_share_list_detail_with_consistency_group(self):
-        env = {'QUERY_STRING': 'name=Share+Test+Name'}
-        req = fakes.HTTPRequest.blank('/shares/detail', environ=env,
-                                      version="2.4")
-        expected = self._list_detail_common_expected()
-        expected['shares'][0]['consistency_group_id'] = None
-        expected['shares'][0]['source_cgsnapshot_member_id'] = None
-        self._list_detail_test_common(req, expected)
-
     def test_share_list_detail_with_task_state(self):
         env = {'QUERY_STRING': 'name=Share+Test+Name'}
         req = fakes.HTTPRequest.blank('/shares/detail', environ=env,
                                       version="2.5")
         expected = self._list_detail_common_expected()
-        expected['shares'][0]['consistency_group_id'] = None
-        expected['shares'][0]['source_cgsnapshot_member_id'] = None
         expected['shares'][0]['task_state'] = None
         self._list_detail_test_common(req, expected)
 
@@ -771,20 +844,6 @@ class ShareAPITest(test.TestCase):
         common.remove_invalid_options(ctx, search_opts, allowed_opts)
         self.assertEqual(expected_opts, search_opts)
 
-    def test_validate_cephx_id_invalid_with_period(self):
-        self.assertRaises(webob.exc.HTTPBadRequest,
-                          self.controller._validate_cephx_id,
-                          "client.manila")
-
-    def test_validate_cephx_id_invalid_with_non_ascii(self):
-        self.assertRaises(webob.exc.HTTPBadRequest,
-                          self.controller._validate_cephx_id,
-                          u"bj\u00F6rn")
-
-    @ddt.data("alice", "alice_bob", "alice bob")
-    def test_validate_cephx_id_valid(self, test_id):
-        self.controller._validate_cephx_id(test_id)
-
 
 def _fake_access_get(self, ctxt, access_id):
 
@@ -809,16 +868,19 @@ def _fake_access_get(self, ctxt, access_id):
 class ShareActionsTest(test.TestCase):
 
     def setUp(self):
-        super(self.__class__, self).setUp()
+        super(ShareActionsTest, self).setUp()
         self.controller = shares.ShareController()
         self.mock_object(share_api.API, 'get', stubs.stub_share_get)
+        self.mock_policy_check = self.mock_object(policy, 'check_policy')
 
     @ddt.data(
         {'access_type': 'ip', 'access_to': '127.0.0.1'},
         {'access_type': 'user', 'access_to': '1' * 4},
-        {'access_type': 'user', 'access_to': '1' * 32},
-        {'access_type': 'user', 'access_to': 'fake\\]{.-_\'`;}['},
-        {'access_type': 'user', 'access_to': 'MYDOMAIN\\Administrator'},
+        {'access_type': 'user', 'access_to': '1' * 255},
+        {'access_type': 'user', 'access_to': 'fake{.-_\'`}'},
+        {'access_type': 'user', 'access_to': 'MYDOMAIN-Administrator'},
+        {'access_type': 'user', 'access_to': 'test group name'},
+        {'access_type': 'user', 'access_to': 'group$.-_\'`{}'},
         {'access_type': 'cert', 'access_to': 'x'},
         {'access_type': 'cert', 'access_to': 'tenant.example.com'},
         {'access_type': 'cert', 'access_to': 'x' * 64},
@@ -835,8 +897,12 @@ class ShareActionsTest(test.TestCase):
         body = {'os-allow_access': access}
         expected = {'access': {'fake': 'fake'}}
         req = fakes.HTTPRequest.blank('/v1/tenant1/shares/%s/action' % id)
+
         res = self.controller._allow_access(req, id, body)
+
         self.assertEqual(expected, res)
+        self.mock_policy_check.assert_called_once_with(
+            req.environ['manila.context'], 'share', 'allow_access')
 
     @ddt.data(
         {'access_type': 'error_type', 'access_to': '127.0.0.1'},
@@ -846,8 +912,10 @@ class ShareActionsTest(test.TestCase):
         {'access_type': 'ip', 'access_to': '127.0.0.256'},
         {'access_type': 'user', 'access_to': '1'},
         {'access_type': 'user', 'access_to': '1' * 3},
-        {'access_type': 'user', 'access_to': '1' * 33},
-        {'access_type': 'user', 'access_to': 'root^'},
+        {'access_type': 'user', 'access_to': '1' * 256},
+        {'access_type': 'user', 'access_to': 'root<>'},
+        {'access_type': 'user', 'access_to': 'group\\'},
+        {'access_type': 'user', 'access_to': '+=*?group'},
         {'access_type': 'cert', 'access_to': ''},
         {'access_type': 'cert', 'access_to': ' '},
         {'access_type': 'cert', 'access_to': 'x' * 65},
@@ -857,8 +925,11 @@ class ShareActionsTest(test.TestCase):
         id = 'fake_share_id'
         body = {'os-allow_access': access}
         req = fakes.HTTPRequest.blank('/v1/tenant1/shares/%s/action' % id)
+
         self.assertRaises(webob.exc.HTTPBadRequest,
                           self.controller._allow_access, req, id, body)
+        self.mock_policy_check.assert_called_once_with(
+            req.environ['manila.context'], 'share', 'allow_access')
 
     def test_deny_access(self):
         def _stub_deny_access(*args, **kwargs):
@@ -870,8 +941,12 @@ class ShareActionsTest(test.TestCase):
         id = 'fake_share_id'
         body = {"os-deny_access": {"access_id": 'fake_acces_id'}}
         req = fakes.HTTPRequest.blank('/v1/tenant1/shares/%s/action' % id)
+
         res = self.controller._deny_access(req, id, body)
+
         self.assertEqual(202, res.status_int)
+        self.mock_policy_check.assert_called_once_with(
+            req.environ['manila.context'], 'share', 'deny_access')
 
     def test_deny_access_not_found(self):
         def _stub_deny_access(*args, **kwargs):
@@ -883,11 +958,29 @@ class ShareActionsTest(test.TestCase):
         id = 'super_fake_share_id'
         body = {"os-deny_access": {"access_id": 'fake_acces_id'}}
         req = fakes.HTTPRequest.blank('/v1/tenant1/shares/%s/action' % id)
+
         self.assertRaises(webob.exc.HTTPNotFound,
                           self.controller._deny_access,
                           req,
                           id,
                           body)
+        self.mock_policy_check.assert_called_once_with(
+            req.environ['manila.context'], 'share', 'deny_access')
+
+    @ddt.data('_allow_access', '_deny_access')
+    def test_allow_access_deny_access_policy_not_authorized(self, method):
+        req = fakes.HTTPRequest.blank('/v1/tenant1/shares/someuuid/action')
+        action = method[1:]
+        body = {action: None}
+        noauthexc = exception.PolicyNotAuthorized(action=action)
+        with mock.patch.object(
+                policy, 'check_policy', mock.Mock(side_effect=noauthexc)):
+            method = getattr(self.controller, method)
+
+            self.assertRaises(
+                webob.exc.HTTPForbidden, method, req, body, 'someuuid')
+            policy.check_policy.assert_called_once_with(
+                req.environ['manila.context'], 'share', action)
 
     def test_access_list(self):
         fake_access_list = [
@@ -997,9 +1090,9 @@ class ShareActionsTest(test.TestCase):
 class ShareAdminActionsAPITest(test.TestCase):
 
     def setUp(self):
-        super(self.__class__, self).setUp()
+        super(ShareAdminActionsAPITest, self).setUp()
         CONF.set_default("default_share_type", None)
-        self.flags(rpc_backend='manila.openstack.common.rpc.impl_fake')
+        self.flags(transport_url='rabbit://fake:fake@mqhost:5672')
         self.share_api = share_api.API()
         self.admin_context = context.RequestContext('admin', 'fake', True)
         self.member_context = context.RequestContext('fake', 'fake')
