@@ -17,14 +17,10 @@
 Tests For Scheduler Manager
 """
 
-try:
-    # Python3 variant
-    from importlib import reload
-except ImportError:
-    pass
+from importlib import reload
+from unittest import mock
 
 import ddt
-import mock
 from oslo_config import cfg
 
 from manila.common import constants
@@ -73,8 +69,8 @@ class SchedulerManagerTestCase(test.TestCase):
         self.fake_args = (1, 2, 3)
         self.fake_kwargs = {'cat': 'meow', 'dog': 'woof'}
 
-    def raise_no_valid_host(*args, **kwargs):
-            raise exception.NoValidHost(reason="")
+    def raise_no_valid_host(self, *args, **kwargs):
+        raise exception.NoValidHost(reason="")
 
     def test_1_correct_init(self):
         # Correct scheduler driver
@@ -224,7 +220,8 @@ class SchedulerManagerTestCase(test.TestCase):
 
         result = self.manager.get_pools(self.context, filters='fake_filters')
 
-        mock_get_pools.assert_called_once_with(self.context, 'fake_filters')
+        mock_get_pools.assert_called_once_with(self.context, 'fake_filters',
+                                               False)
         self.assertEqual('fake_pools', result)
 
     @mock.patch.object(db, 'share_group_update', mock.Mock())
