@@ -15,8 +15,9 @@
 
 """Test cases for GlusterFS common routines."""
 
+from unittest import mock
+
 import ddt
-import mock
 from oslo_config import cfg
 
 from manila import exception
