@@ -19,9 +19,9 @@
 import re
 import shutil
 import tempfile
+from unittest import mock
 
 import ddt
-import mock
 from oslo_config import cfg
 
 from manila.common import constants
@@ -105,7 +105,7 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
         CONF.set_default('glusterfs_path_to_private_key',
                          '/fakepath/to/privatekey')
         CONF.set_default('glusterfs_volume_pattern',
-                         'manila-share-\d+-#{size}G$')
+                         r'manila-share-\d+-#{size}G$')
         CONF.set_default('driver_handles_share_servers', False)
 
         self.fake_driver = mock.Mock()
@@ -151,7 +151,7 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
         self.assertEqual(fake_obj, ret)
 
     def test_compile_volume_pattern(self):
-        volume_pattern = 'manila-share-\d+-(?P<size>\d+)G$'
+        volume_pattern = r'manila-share-\d+-(?P<size>\d+)G$'
 
         ret = self._layout._compile_volume_pattern()
 
@@ -242,7 +242,7 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
         self.mock_object(self._layout, '_glustermanager',
                          mock.Mock(side_effect=_glustermanager_calls))
         self.mock_object(self._layout, 'volume_pattern',
-                         re.compile('manila-share-\d+(-(?P<size>\d+)G)?$'))
+                         re.compile(r'manila-share-\d+(-(?P<size>\d+)G)?$'))
         expected_output = {'root@host1:/manila-share-1': {'size': None}}
 
         ret = self._layout._fetch_gluster_volumes()
@@ -587,6 +587,7 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
         gmgr1 = gmgr(self.glusterfs_target1, self._execute, None, None)
         gmgr1.set_vol_option = mock.Mock()
         gmgr1.get_vol_option = mock.Mock(return_value=clone_of)
+        new_vol_addr = self.glusterfs_target1
         self.mock_object(self._layout, '_glustermanager',
                          mock.Mock(return_value=gmgr1))
         self._layout.gluster_used_vols = set([self.glusterfs_target1])
@@ -596,12 +597,15 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
         gmgr1.get_vol_option.assert_called_once_with(
             'user.manila-cloned-from')
         self._layout._wipe_gluster_vol.assert_called_once_with(gmgr1)
+        self.assertIn(new_vol_addr, self._layout.gluster_used_vols)
         self._layout._push_gluster_vol.assert_called_once_with(
             self.glusterfs_target1)
         self._layout.private_storage.delete.assert_called_once_with(
             self.share1['id'])
-        gmgr1.set_vol_option.assert_called_once_with(
-            'user.manila-share', 'NONE')
+        gmgr1.set_vol_option.assert_has_calls([
+            mock.call('user.manila-share', 'NONE'),
+            mock.call('nfs.disable', 'on')
+        ])
 
     def test_delete_share_clone(self):
         self._layout._push_gluster_vol = mock.Mock()
