@@ -14,9 +14,9 @@
 
 from copy import deepcopy
 import sys
+from unittest import mock
 
 import ddt
-import mock
 if 'hpe3parclient' not in sys.modules:
     sys.modules['hpe3parclient'] = mock.Mock()
 
@@ -30,9 +30,6 @@ from manila.tests.share.drivers.hpe import test_hpe_3par_constants as constants
 @ddt.ddt
 class HPE3ParDriverFPGTestCase(test.TestCase):
 
-    def setUp(self):
-        super(HPE3ParDriverFPGTestCase, self).setUp()
-
     @ddt.data((-1, 4),
               (0, 5),
               (0, -1))
@@ -117,10 +114,11 @@ class HPE3ParDriverTestCase(test.TestCase):
         self.driver = hpe3pardriver.HPE3ParShareDriver(
             configuration=self.conf)
 
-    def test_driver_setup_success(self):
+    def test_driver_setup_success(self,
+                                  get_vfs_ret_val=constants.EXPECTED_GET_VFS):
         """Driver do_setup without any errors."""
 
-        self.mock_mediator.get_vfs.return_value = constants.EXPECTED_GET_VFS
+        self.mock_mediator.get_vfs.return_value = get_vfs_ret_val
 
         self.driver.do_setup(None)
         conf = self.conf
@@ -162,6 +160,15 @@ class HPE3ParDriverTestCase(test.TestCase):
         self.test_driver_setup_success()
         self.assertEqual(constants.EXPECTED_FPG_MAP, self.driver.fpgs)
 
+    def test_driver_setup_no_dhss_multi_getvfs_success(self):
+        """Driver do_setup when dhss=False, getvfs returns multiple IPs."""
+
+        self.conf.driver_handles_share_servers = False
+        self.test_driver_setup_success(
+            get_vfs_ret_val=constants.EXPECTED_GET_VFS_MULTIPLES)
+        self.assertEqual(constants.EXPECTED_FPG_MAP,
+                         self.driver.fpgs)
+
     def test_driver_setup_success_no_dhss_no_conf_ss_ip(self):
         """test driver's do_setup()
 
@@ -177,7 +184,7 @@ class HPE3ParDriverTestCase(test.TestCase):
         self.test_driver_setup_success()
 
         self.assertEqual(constants.EXPECTED_FPG_MAP, self.driver.fpgs)
-        self.conf.hpe3par_fpg = original_fpg
+        constants.EXPECTED_FPG_CONF = original_fpg
 
     def test_driver_setup_failure_no_dhss_no_conf_ss_ip(self):
         """Configured IP address is required for dhss=False."""
@@ -193,7 +200,7 @@ class HPE3ParDriverTestCase(test.TestCase):
 
         self.assertRaises(exception.HPE3ParInvalid,
                           self.driver.do_setup, None)
-        self.conf.hpe3par_fpg = fpg_without_ss_ip
+        constants.EXPECTED_FPG_CONF = fpg_without_ss_ip
 
     def test_driver_setup_mediator_error(self):
         """Driver do_setup when the mediator setup fails."""
@@ -431,8 +438,8 @@ class HPE3ParDriverTestCase(test.TestCase):
     def test_driver_create_cifs_share(self):
         self.init_driver()
 
-        expected_location = '\\\\%s\%s' % (constants.EXPECTED_IP_10203040,
-                                           constants.EXPECTED_SHARE_NAME)
+        expected_location = '\\\\%s\\%s' % (constants.EXPECTED_IP_10203040,
+                                            constants.EXPECTED_SHARE_NAME)
 
         self.mock_mediator.create_share.return_value = (
             constants.EXPECTED_SHARE_NAME)
@@ -489,8 +496,8 @@ class HPE3ParDriverTestCase(test.TestCase):
     def test_driver_create_cifs_share_from_snapshot(self):
         self.init_driver()
 
-        expected_location = '\\\\%s\%s' % (constants.EXPECTED_IP_10203040,
-                                           constants.EXPECTED_SHARE_NAME)
+        expected_location = '\\\\%s\\%s' % (constants.EXPECTED_IP_10203040,
+                                            constants.EXPECTED_SHARE_NAME)
 
         self.mock_mediator.create_share_from_snapshot.return_value = (
             constants.EXPECTED_SHARE_NAME)
@@ -723,6 +730,12 @@ class HPE3ParDriverTestCase(test.TestCase):
             'provisioned_capacity_gb': 0,
             'share_backend_name': 'HPE_3PAR',
             'snapshot_support': True,
+            'create_share_from_snapshot_support': True,
+            'revert_to_snapshot_support': False,
+            'mount_snapshot_support': False,
+            'share_group_stats': {
+                'consistent_snapshot_support': None,
+            },
             'storage_protocol': 'NFS_CIFS',
             'thin_provisioning': True,
             'total_capacity_gb': 0,
@@ -731,6 +744,8 @@ class HPE3ParDriverTestCase(test.TestCase):
             'replication_domain': None,
             'filter_function': None,
             'goodness_function': None,
+            'ipv4_support': True,
+            'ipv6_support': False,
         }
 
         result = self.driver.get_share_stats(refresh=True)
@@ -797,9 +812,17 @@ class HPE3ParDriverTestCase(test.TestCase):
                 'reserved_percentage': 0,
                 'provisioned_capacity_gb': expected_capacity}],
             'snapshot_support': True,
+            'create_share_from_snapshot_support': True,
+            'revert_to_snapshot_support': False,
+            'mount_snapshot_support': False,
+            'share_group_stats': {
+                'consistent_snapshot_support': None,
+            },
             'replication_domain': None,
             'filter_function': None,
             'goodness_function': None,
+            'ipv4_support': True,
+            'ipv6_support': False,
         }
 
         result = self.driver.get_share_stats(refresh=True)
@@ -833,9 +856,17 @@ class HPE3ParDriverTestCase(test.TestCase):
             'total_capacity_gb': 0,
             'vendor_name': 'HPE',
             'snapshot_support': True,
+            'create_share_from_snapshot_support': True,
+            'revert_to_snapshot_support': False,
+            'mount_snapshot_support': False,
+            'share_group_stats': {
+                'consistent_snapshot_support': None,
+            },
             'replication_domain': None,
             'filter_function': None,
             'goodness_function': None,
+            'ipv4_support': True,
+            'ipv6_support': False,
         }
 
         result = self.driver.get_share_stats(refresh=True)
