@@ -16,7 +16,8 @@
 
 import copy
 import functools
-import mock
+from unittest import mock
+
 from oslo_utils import units
 
 from manila.common import constants
@@ -346,11 +347,11 @@ class InfiniboxDriverTestCase(InfiniboxDriverTestCaseBase):
             self._mock_export.get_export_path())
         for network_space_ip in \
                 (_MOCK_NETWORK_SPACE_IP_1, _MOCK_NETWORK_SPACE_IP_2):
-                self.assertIn(
-                    "{network_space_ip}:{export_path}".format(
-                        network_space_ip=network_space_ip,
-                        export_path=self._mock_export.get_export_path()),
-                    export_paths)
+            self.assertIn(
+                "{network_space_ip}:{export_path}".format(
+                    network_space_ip=network_space_ip,
+                    export_path=self._mock_export.get_export_path()),
+                export_paths)
 
     def test__get_export(self):
         # The default return value of get_exports is [mock_export, ]:
