@@ -12,9 +12,10 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from unittest import mock
+
 from lxml import etree
-import mock
-from six.moves import urllib
+import requests
 
 from manila.share.drivers.netapp.dataontap.client import api
 
@@ -73,6 +74,18 @@ DELETED_EXPORT_POLICIES = {
 }
 QOS_POLICY_GROUP_NAME = 'fake_qos_policy_group_name'
 QOS_MAX_THROUGHPUT = '5000B/s'
+ADAPTIVE_QOS_POLICY_GROUP_NAME = 'fake_adaptive_qos_policy_group_name'
+VSERVER_TYPE_DEFAULT = 'default'
+VSERVER_TYPE_DP_DEST = 'dp_destination'
+VSERVER_OP_STATE_RUNNING = 'running'
+VSERVER_STATE = 'running'
+VSERVER_INFO = {
+    'name': VSERVER_NAME,
+    'subtype': VSERVER_TYPE_DEFAULT,
+    'operational_state': VSERVER_OP_STATE_RUNNING,
+    'state': VSERVER_STATE,
+}
+SNAPMIRROR_POLICY_NAME = 'fake_snapmirror_policy'
 
 USER_NAME = 'fake_user'
 
@@ -155,6 +168,15 @@ QOS_POLICY_GROUP = {
     'num-workloads': 1,
 }
 
+VOLUME_AUTOSIZE_ATTRS = {
+    'mode': 'off',
+    'grow-threshold-percent': '85',
+    'shrink-threshold-percent': '50',
+    'maximum-size': '1258288',
+    'minimum-size': '1048576',
+}
+
+
 NO_RECORDS_RESPONSE = etree.XML("""
   <results status="passed">
     <num-records>0</num-records>
@@ -197,6 +219,20 @@ VSERVER_GET_ITER_RESPONSE = etree.XML("""
   </results>
 """ % {'fake_vserver': VSERVER_NAME})
 
+VSERVER_GET_ITER_RESPONSE_INFO = etree.XML("""
+  <results status="passed">
+    <attributes-list>
+      <vserver-info>
+        <operational-state>%(operational_state)s</operational-state>
+        <state>%(state)s</state>
+        <vserver-name>%(name)s</vserver-name>
+        <vserver-subtype>%(subtype)s</vserver-subtype>
+      </vserver-info>
+    </attributes-list>
+    <num-records>1</num-records>
+  </results>
+""" % VSERVER_INFO)
+
 VSERVER_GET_ROOT_VOLUME_NAME_RESPONSE = etree.XML("""
   <results status="passed">
     <attributes-list>
@@ -1701,6 +1737,18 @@ CIFS_SHARE_ACCESS_CONTROL_GET_ITER = etree.XML("""
   </results>
 """ % {'volume': SHARE_NAME})
 
+CIFS_SHARE_GET_ITER_RESPONSE = etree.XML("""
+ <results status="passed">
+    <attributes-list>
+      <cifs-share>
+        <share-name>%(share_name)s</share-name>
+        <vserver>fake_vserver</vserver>
+      </cifs-share>
+    </attributes-list>
+    <num-records>1</num-records>
+  </results>
+""" % {'share_name': SHARE_NAME})
+
 NFS_EXPORT_RULES = ('10.10.10.10', '10.10.10.20')
 
 NFS_EXPORTFS_LIST_RULES_2_NO_RULES_RESPONSE = etree.XML("""
@@ -1969,6 +2017,22 @@ GET_AGGREGATE_FOR_VOLUME_RESPONSE = etree.XML("""
     'share': SHARE_NAME
 })
 
+VOLUME_AUTOSIZE_GET_RESPONSE = etree.XML("""
+  <results status="passed">
+    <grow-threshold-percent>%(grow_percent)s</grow-threshold-percent>
+    <is-enabled>false</is-enabled>
+    <maximum-size>%(max_size)s</maximum-size>
+    <minimum-size>%(min_size)s</minimum-size>
+    <mode>%(mode)s</mode>
+    <shrink-threshold-percent>%(shrink_percent)s</shrink-threshold-percent>
+  </results>
+""" % {'grow_percent': VOLUME_AUTOSIZE_ATTRS.get('grow-threshold-percent'),
+       'max_size': VOLUME_AUTOSIZE_ATTRS.get('maximum-size'),
+       'min_size': VOLUME_AUTOSIZE_ATTRS.get('minimum-size'),
+       'mode': VOLUME_AUTOSIZE_ATTRS.get('mode'),
+       'shrink_percent': VOLUME_AUTOSIZE_ATTRS.get(
+           'shrink-threshold-percent')})
+
 GET_VOLUME_FOR_ENCRYPTED_RESPONSE = etree.XML("""
   <results status="passed">
     <attributes-list>
@@ -2195,6 +2259,46 @@ VOLUME_GET_ITER_CLONE_CHILDREN_RESPONSE = etree.XML("""
     'clone2': CLONE_CHILD_2,
 })
 
+VOLUME_GET_ITER_PARENT_SNAP_EMPTY_RESPONSE = etree.XML("""
+  <results status="passed">
+    <attributes-list>
+      <volume-attributes>
+        <volume-id-attributes>
+          <name>%(name)s</name>
+          <owning-vserver-name>%(vserver)s</owning-vserver-name>
+        </volume-id-attributes>
+      </volume-attributes>
+    </attributes-list>
+    <num-records>1</num-records>
+  </results>
+""" % {
+    'vserver': VSERVER_NAME,
+    'name': SHARE_NAME,
+})
+
+VOLUME_GET_ITER_PARENT_SNAP_RESPONSE = etree.XML("""
+  <results status="passed">
+    <attributes-list>
+      <volume-attributes>
+        <volume-clone-attributes>
+          <volume-clone-parent-attributes>
+            <snapshot-name>%(snapshot_name)s</snapshot-name>
+          </volume-clone-parent-attributes>
+        </volume-clone-attributes>
+        <volume-id-attributes>
+          <name>%(name)s</name>
+          <owning-vserver-name>%(vserver)s</owning-vserver-name>
+        </volume-id-attributes>
+      </volume-attributes>
+    </attributes-list>
+    <num-records>1</num-records>
+  </results>
+""" % {
+    'snapshot_name': SNAPSHOT_NAME,
+    'vserver': VSERVER_NAME,
+    'name': SHARE_NAME,
+})
+
 SIS_GET_ITER_RESPONSE = etree.XML("""
   <results status="passed">
     <attributes-list>
@@ -2332,6 +2436,7 @@ SNAPMIRROR_GET_ITER_FILTERED_RESPONSE = etree.XML("""
         <destination-volume>fake_destination_volume</destination-volume>
         <is-healthy>true</is-healthy>
         <mirror-state>snapmirrored</mirror-state>
+        <relationship-status>idle</relationship-status>
         <schedule>daily</schedule>
         <source-vserver>fake_source_vserver</source-vserver>
         <source-volume>fake_source_volume</source-volume>
@@ -2341,6 +2446,35 @@ SNAPMIRROR_GET_ITER_FILTERED_RESPONSE = etree.XML("""
   </results>
 """)
 
+SNAPMIRROR_GET_ITER_FILTERED_RESPONSE_2 = etree.XML("""
+  <results status="passed">
+    <attributes-list>
+      <snapmirror-info>
+        <source-vserver>fake_source_vserver</source-vserver>
+        <destination-vserver>fake_destination_vserver</destination-vserver>
+        <mirror-state>snapmirrored</mirror-state>
+        <relationship-status>idle</relationship-status>
+      </snapmirror-info>
+    </attributes-list>
+    <num-records>1</num-records>
+  </results>
+""")
+
+SNAPMIRROR_GET_DESTINATIONS_ITER_FILTERED_RESPONSE = etree.XML("""
+  <results status="passed">
+    <attributes-list>
+      <snapmirror-destination-info>
+        <destination-location>fake_destination_vserver:</destination-location>
+        <destination-vserver>fake_destination_vserver</destination-vserver>
+        <relationship-id>fake_relationship_id</relationship-id>
+        <source-location>fake_source_vserver:</source-location>
+        <source-vserver>fake_source_vserver</source-vserver>
+      </snapmirror-destination-info>
+    </attributes-list>
+    <num-records>1</num-records>
+  </results>
+""")
+
 SNAPMIRROR_INITIALIZE_RESULT = etree.XML("""
   <results status="passed">
     <result-status>succeeded</result-status>
@@ -2368,6 +2502,43 @@ VOLUME_MOVE_GET_ITER_RESULT = etree.XML("""
     'vserver': VSERVER_NAME,
 })
 
+NFS_INFO_STR = """
+    <nfs-info>
+        <is-rquota-enabled>false</is-rquota-enabled>
+        <is-tcp-enabled>true</is-tcp-enabled>
+        <is-v3-hide-snapshot>false</is-v3-hide-snapshot>
+        <ntfs-unix-security-ops>use_export_policy</ntfs-unix-security-ops>
+        <permitted-enc-types>
+        <string>des</string>
+        <string>des3</string>
+        <string>aes_128</string>
+        <string>aes_256</string>
+        </permitted-enc-types>
+        <tcp-max-xfer-size>65536</tcp-max-xfer-size>
+        <udp-max-xfer-size>32768</udp-max-xfer-size>
+        <v3-search-unconverted-filename>false</v3-search-unconverted-filename>
+        <v4-inherited-acl-preserve>false</v4-inherited-acl-preserve>
+    </nfs-info>
+"""
+
+NFS_INFO_DEFAULT_TREE = etree.XML(NFS_INFO_STR)
+
+NFS_CONFIG_DEFAULT_RESULT = etree.XML("""
+    <results status="passed">
+       <defaults>
+          %s
+        </defaults>
+    </results>
+""" % NFS_INFO_STR)
+
+NFS_CONFIG_SERVER_RESULT = etree.XML("""
+    <results status="passed">
+       <attributes-list>
+        %s
+       </attributes-list>
+    </results>
+""" % NFS_INFO_STR)
+
 PERF_OBJECT_COUNTER_TOTAL_CP_MSECS_LABELS = [
     'SETUP', 'PRE_P0', 'P0_SNAP_DEL', 'P1_CLEAN', 'P1_QUOTA', 'IPU_DISK_ADD',
     'P2V_INOFILE', 'P2V_INO_PUB', 'P2V_INO_PRI', 'P2V_FSINFO', 'P2V_DLOG1',
@@ -2527,6 +2698,20 @@ QOS_POLICY_GROUP_GET_ITER_RESPONSE = etree.XML("""
     'max_throughput': QOS_MAX_THROUGHPUT,
 })
 
+SNAPMIRROR_POLICY_GET_ITER_RESPONSE = etree.XML("""
+  <results status="passed">
+    <attributes-list>
+      <snapmirror-policy-info>
+        <policy-name>%(policy_name)s</policy-name>
+        <vserver-name>%(vserver_name)s</vserver-name>
+      </snapmirror-policy-info>
+    </attributes-list>
+    <num-records>1</num-records>
+  </results>""" % {
+    'policy_name': SNAPMIRROR_POLICY_NAME,
+    'vserver_name': VSERVER_NAME,
+})
+
 FAKE_VOL_XML = """<volume-info>
     <name>open123</name>
     <state>online</state>
@@ -2597,7 +2782,7 @@ FAKE_RESULT_API_ERRNO_VALID.add_attr('errno', '14956')
 FAKE_RESULT_SUCCESS = api.NaElement('result')
 FAKE_RESULT_SUCCESS.add_attr('status', 'passed')
 
-FAKE_HTTP_OPENER = urllib.request.build_opener()
+FAKE_HTTP_SESSION = requests.Session()
 
 FAKE_MANAGE_VOLUME = {
     'aggregate': SHARE_AGGREGATE_NAME,
