@@ -14,16 +14,21 @@
 #    under the License.
 
 import copy
+from oslo_serialization import jsonutils
 
 from manila.common import constants
 import manila.tests.share.drivers.netapp.fakes as na_fakes
 
-
+NO_SNAPRESTORE_LICENSE = '"SnapRestore" is not licensed in the cluster.'
+CLUSTER_NAME = 'fake_cluster'
+CLUSTER_NAME_2 = 'fake_cluster_2'
 BACKEND_NAME = 'fake_backend_name'
+BACKEND_NAME_2 = 'fake_backend_name_2'
 DRIVER_NAME = 'fake_driver_name'
 APP_VERSION = 'fake_app_vsersion'
 HOST_NAME = 'fake_host'
 POOL_NAME = 'fake_pool'
+POOL_NAME_2 = 'fake_pool_2'
 VSERVER1 = 'fake_vserver_1'
 VSERVER2 = 'fake_vserver_2'
 LICENSES = ('base', 'cifs', 'fcp', 'flexclone', 'iscsi', 'nfs', 'snapmirror',
@@ -32,6 +37,7 @@ VOLUME_NAME_TEMPLATE = 'share_%(share_id)s'
 VSERVER_NAME_TEMPLATE = 'os_%s'
 AGGREGATE_NAME_SEARCH_PATTERN = '(.*)'
 SHARE_NAME = 'share_7cf7c200_d3af_4e05_b87e_9167c95dfcad'
+SHARE_INSTANCE_NAME = 'share_d24e7257_124e_4fb6_b05b_d384f660bc85'
 FLEXVOL_NAME = 'fake_volume'
 JUNCTION_PATH = '/%s' % FLEXVOL_NAME
 EXPORT_LOCATION = '%s:%s' % (HOST_NAME, JUNCTION_PATH)
@@ -44,6 +50,7 @@ SHARE_ID = '7cf7c200-d3af-4e05-b87e-9167c95dfcad'
 SHARE_ID2 = 'b51c5a31-aa5b-4254-9ee8-7d39fa4c8c38'
 SHARE_ID3 = '1379991d-037b-4897-bf3a-81b4aac72eff'
 SHARE_ID4 = '1cb41aad-fd9b-4964-8059-646f69de925e'
+SHARE_INSTANCE_ID = 'd24e7257-124e-4fb6-b05b-d384f660bc85'
 PARENT_SHARE_ID = '585c3935-2aa9-437c-8bad-5abae1076555'
 SNAPSHOT_ID = 'de4c9050-e2f9-4ce1-ade4-5ed0c9f26451'
 CONSISTENCY_GROUP_ID = '65bfa2c9-dc6c-4513-951a-b8d15b453ad8'
@@ -51,6 +58,7 @@ CONSISTENCY_GROUP_ID2 = '35f5c1ea-45fb-40c4-98ae-2a2a17554159'
 CG_SNAPSHOT_ID = '6ddd8a6b-5df7-417b-a2ae-3f6e449f4eea'
 CG_SNAPSHOT_MEMBER_ID1 = '629f79ef-b27e-4596-9737-30f084e5ba29'
 CG_SNAPSHOT_MEMBER_ID2 = 'e876aa9c-a322-4391-bd88-9266178262be'
+SERVER_ID = 'd5e90724-6f28-4944-858a-553138bdbd29'
 FREE_CAPACITY = 10000000000
 TOTAL_CAPACITY = 20000000000
 AGGREGATE = 'manila_aggr_1'
@@ -64,6 +72,7 @@ NODE_DATA_PORT = 'e0c'
 NODE_DATA_PORTS = ('e0c', 'e0d')
 LIF_NAME_TEMPLATE = 'os_%(net_allocation_id)s'
 SHARE_TYPE_ID = '26e89a5b-960b-46bb-a8cf-0778e653098f'
+SHARE_TYPE_ID_2 = '2a06887e-25b5-486e-804a-d84c2d806feb'
 SHARE_TYPE_NAME = 'fake_share_type'
 IPSPACE = 'fake_ipspace'
 IPSPACE_ID = '27d38c27-3e8b-4d7d-9d91-fcf295e3ac8f'
@@ -71,6 +80,14 @@ MTU = 1234
 DEFAULT_MTU = 1500
 MANILA_HOST_NAME = '%(host)s@%(backend)s#%(pool)s' % {
     'host': HOST_NAME, 'backend': BACKEND_NAME, 'pool': POOL_NAME}
+MANILA_HOST_NAME_2 = '%(host)s@%(backend)s#%(pool)s' % {
+    'host': HOST_NAME, 'backend': BACKEND_NAME, 'pool': POOL_NAME_2}
+MANILA_HOST_NAME_3 = '%(host)s@%(backend)s#%(pool)s' % {
+    'host': HOST_NAME, 'backend': BACKEND_NAME_2, 'pool': POOL_NAME_2}
+SERVER_HOST = '%(host)s@%(backend)s' % {
+    'host': HOST_NAME, 'backend': BACKEND_NAME}
+SERVER_HOST_2 = '%(host)s@%(backend)s' % {
+    'host': HOST_NAME, 'backend': BACKEND_NAME_2}
 QOS_EXTRA_SPEC = 'netapp:maxiops'
 QOS_SIZE_DEPENDENT_EXTRA_SPEC = 'netapp:maxbpspergib'
 QOS_NORMALIZED_SPEC = 'maxiops'
@@ -106,6 +123,21 @@ SHARE = {
     'encrypt': False,
 }
 
+SHARE_INSTANCE = {
+    'id': SHARE_INSTANCE_ID,
+    'share_id': SHARE_ID,
+    'host': MANILA_HOST_NAME,
+    'project_id': TENANT_ID,
+    'name': SHARE_INSTANCE_NAME,
+    'size': SHARE_SIZE,
+    'share_proto': 'fake',
+    'share_type_id': SHARE_TYPE_ID,
+    'share_network_id': '5dfe0898-e2a1-4740-9177-81c7d26713b0',
+    'share_server_id': '7e6a2cc8-871f-4b1d-8364-5aad0f98da86',
+    'replica_state': constants.REPLICA_STATE_ACTIVE,
+    'status': constants.STATUS_AVAILABLE,
+}
+
 FLEXVOL_TO_MANAGE = {
     'aggregate': POOL_NAME,
     'junction-path': '/%s' % FLEXVOL_NAME,
@@ -137,6 +169,15 @@ FLEXVOL = {
     'owning-vserver-name': VSERVER1,
 }
 
+SHARE_TYPE_EXTRA_SPEC = {
+    'snapshot_support': True,
+    'create_share_from_snapshot_support': True,
+    'revert_to_snapshot_support': True,
+    'mount_snapshot_support': False,
+    'driver_handles_share_servers': True,
+    'availability_zones': [],
+}
+
 EXTRA_SPEC = {
     'netapp:thin_provisioned': 'true',
     'netapp:snapshot_policy': 'default',
@@ -148,6 +189,70 @@ EXTRA_SPEC = {
     'netapp_disk_type': 'FCAL',
     'netapp_raid_type': 'raid4',
     'netapp_flexvol_encryption': 'true',
+    'netapp:tcp_max_xfer_size': 100,
+    'netapp:udp_max_xfer_size': 100,
+    'netapp:adaptive_qos_policy_group': None,
+}
+
+EXTRA_SPEC_WITH_REPLICATION = copy.copy(EXTRA_SPEC)
+EXTRA_SPEC_WITH_REPLICATION.update({
+    'replication_type': 'dr'
+})
+
+NFS_CONFIG_DEFAULT = {
+    'tcp-max-xfer-size': 65536,
+    'udp-max-xfer-size': 32768,
+}
+
+NFS_CONFIG_TCP_MAX_DDT = {
+    'extra_specs': {
+        'netapp:tcp_max_xfer_size': 100,
+    },
+    'expected': {
+        'tcp-max-xfer-size': 100,
+        'udp-max-xfer-size': NFS_CONFIG_DEFAULT['udp-max-xfer-size'],
+    }
+}
+
+NFS_CONFIG_UDP_MAX_DDT = {
+    'extra_specs': {
+        'netapp:udp_max_xfer_size': 100,
+    },
+    'expected': {
+        'tcp-max-xfer-size': NFS_CONFIG_DEFAULT['tcp-max-xfer-size'],
+        'udp-max-xfer-size': 100,
+    }
+}
+
+NFS_CONFIG_TCP_UDP_MAX = {
+    'tcp-max-xfer-size': 100,
+    'udp-max-xfer-size': 100,
+}
+
+NFS_CONFIG_TCP_MAX = {
+    'tcp-max-xfer-size': 100,
+    'udp-max-xfer-size': NFS_CONFIG_DEFAULT['udp-max-xfer-size'],
+}
+
+NFS_CONFIG_UDP_MAX = {
+    'tcp-max-xfer-size': NFS_CONFIG_DEFAULT['tcp-max-xfer-size'],
+    'udp-max-xfer-size': 100,
+}
+
+NFS_CONFIG_TCP_UDP_MAX_DDT = {
+    'extra_specs': {
+        'netapp:tcp_max_xfer_size': 100,
+        'netapp:udp_max_xfer_size': 100,
+    },
+    'expected': NFS_CONFIG_TCP_UDP_MAX,
+}
+
+SHARE_GROUP_REF = {
+    'share_types': [
+        {'share_type_id': 'id1'},
+        {'share_type_id': 'id2'},
+        {'share_type_id': 'id3'},
+    ],
 }
 
 EXTRA_SPEC_WITH_QOS = copy.deepcopy(EXTRA_SPEC)
@@ -172,12 +277,17 @@ PROVISIONING_OPTIONS = {
     'split': True,
     'encrypt': False,
     'hide_snapdir': False,
+    'adaptive_qos_policy_group': None,
 }
 
 PROVISIONING_OPTIONS_WITH_QOS = copy.deepcopy(PROVISIONING_OPTIONS)
 PROVISIONING_OPTIONS_WITH_QOS.update(
     {'qos_policy_group': QOS_POLICY_GROUP_NAME})
 
+PROVISIONING_OPTS_WITH_ADAPT_QOS = copy.deepcopy(PROVISIONING_OPTIONS)
+PROVISIONING_OPTS_WITH_ADAPT_QOS.update(
+    {'adaptive_qos_policy_group': QOS_POLICY_GROUP_NAME})
+
 PROVISIONING_OPTIONS_BOOLEAN = {
     'thin_provisioned': True,
     'dedup_enabled': False,
@@ -201,18 +311,21 @@ PROVISIONING_OPTIONS_STRING = {
     'snapshot_policy': 'default',
     'language': 'en-US',
     'max_files': 5000,
+    'adaptive_qos_policy_group': None,
 }
 
 PROVISIONING_OPTIONS_STRING_MISSING_SPECS = {
     'snapshot_policy': 'default',
     'language': 'en-US',
     'max_files': None,
+    'adaptive_qos_policy_group': None,
 }
 
 PROVISIONING_OPTIONS_STRING_DEFAULT = {
     'snapshot_policy': None,
     'language': None,
     'max_files': None,
+    'adaptive_qos_policy_group': None,
 }
 
 SHORT_BOOLEAN_EXTRA_SPEC = {
@@ -223,11 +336,13 @@ STRING_EXTRA_SPEC = {
     'netapp:snapshot_policy': 'default',
     'netapp:language': 'en-US',
     'netapp:max_files': 5000,
+    'netapp:adaptive_qos_policy_group': None,
 }
 
 SHORT_STRING_EXTRA_SPEC = {
     'netapp:snapshot_policy': 'default',
     'netapp:language': 'en-US',
+    'netapp:adaptive_qos_policy_group': None,
 }
 
 INVALID_EXTRA_SPEC = {
@@ -245,6 +360,14 @@ INVALID_MAX_FILE_EXTRA_SPEC = {
     'netapp:max_files': -1,
 }
 
+INVALID_TCP_MAX_XFER_SIZE_EXTRA_SPEC = {
+    'netapp:tcp_max_xfer_size': -1,
+}
+
+INVALID_UDP_MAX_XFER_SIZE_EXTRA_SPEC = {
+    'netapp:udp_max_xfer_size': -1,
+}
+
 EMPTY_EXTRA_SPEC = {}
 
 SHARE_TYPE = {
@@ -268,8 +391,13 @@ REMAPPED_OVERLAPPING_EXTRA_SPEC = {
     'netapp:thin_provisioned': 'false',
 }
 
-EXTRA_SPEC_SHARE = copy.deepcopy(SHARE)
-EXTRA_SPEC_SHARE['share_type_id'] = SHARE_TYPE_ID
+VOLUME_AUTOSIZE_ATTRS = {
+    'mode': 'off',
+    'grow-threshold-percent': '85',
+    'shrink-threshold-percent': '50',
+    'maximum-size': '1258288',
+    'minimum-size': '1048576',
+}
 
 USER_NETWORK_ALLOCATIONS = [
     {
@@ -335,20 +463,106 @@ NETWORK_INFO = {
     'security_services': ['fake_ldap', 'fake_kerberos', 'fake_ad', ],
     'network_allocations': USER_NETWORK_ALLOCATIONS,
     'admin_network_allocations': ADMIN_NETWORK_ALLOCATIONS,
+    'neutron_net_id': '4eff22ca-5ad2-454d-a000-aadfd7b40b39',
     'neutron_subnet_id': '62bf1c2c-18eb-421b-8983-48a6d39aafe0',
     'segmentation_id': '1000',
 }
 NETWORK_INFO_NETMASK = '255.255.255.0'
 
 SHARE_SERVER = {
+    'id': 'fake_id',
     'share_network_id': 'c5b3a865-56d0-4d88-abe5-879965e099c9',
     'backend_details': {
         'vserver_name': VSERVER1
     },
     'network_allocations': (USER_NETWORK_ALLOCATIONS +
                             ADMIN_NETWORK_ALLOCATIONS),
+    'host': SERVER_HOST,
 }
 
+SHARE_SERVER_2 = {
+    'id': 'fake_id_2',
+    'share_network_id': 'c5b3a865-56d0-4d88-abe5-879965e099c9',
+    'backend_details': {
+        'vserver_name': VSERVER2
+    },
+    'network_allocations': (USER_NETWORK_ALLOCATIONS +
+                            ADMIN_NETWORK_ALLOCATIONS),
+    'host': SERVER_HOST_2,
+}
+
+VSERVER_INFO = {
+    'name': 'fake_vserver_name',
+    'subtype': 'default',
+    'operational_state': 'running',
+    'state': 'running',
+}
+
+SHARE_SERVER_NFS_TCP = {
+    'id': 'fake_nfs_id_tcp',
+    'backend_details': {
+        'vserver_name': VSERVER2,
+        'nfs_config': jsonutils.dumps(NFS_CONFIG_TCP_MAX),
+    },
+    'host': 'fake_host@fake_backend',
+}
+
+SHARE_SERVER_NFS_UDP = {
+    'id': 'fake_nfs_id_udp',
+    'backend_details': {
+        'vserver_name': VSERVER2,
+        'nfs_config': jsonutils.dumps(NFS_CONFIG_UDP_MAX),
+    },
+    'host': 'fake_host@fake_backend',
+}
+
+SHARE_SERVER_NFS_TCP_UDP = {
+    'id': 'fake_nfs_id_tcp_udp',
+    'backend_details': {
+        'vserver_name': VSERVER2,
+        'nfs_config': jsonutils.dumps(NFS_CONFIG_TCP_UDP_MAX),
+    },
+    'host': 'fake_host@fake_backend',
+}
+
+SHARE_SERVER_NO_NFS_NONE = {
+    'id': 'fake_no_nfs_id_none',
+    'backend_details': {
+        'vserver_name': VSERVER2,
+    },
+    'host': 'fake_host@fake_backend',
+}
+
+SHARE_SERVER_NO_DETAILS = {
+    'id': 'id_no_datails',
+    'host': 'fake_host@fake_backend',
+}
+
+SHARE_SERVER_NFS_DEFAULT = {
+    'id': 'fake_id_nfs_default',
+    'backend_details': {
+        'vserver_name': VSERVER2,
+        'nfs_config': jsonutils.dumps(NFS_CONFIG_DEFAULT),
+    },
+    'host': 'fake_host@fake_backend',
+}
+
+SHARE_SERVERS = [
+    SHARE_SERVER_NFS_TCP,
+    SHARE_SERVER_NFS_UDP,
+    SHARE_SERVER_NFS_TCP_UDP,
+    SHARE_SERVER_NFS_DEFAULT,
+    SHARE_SERVER_NO_NFS_NONE,
+    SHARE_SERVER_NO_DETAILS,
+]
+
+VSERVER_PEER = [{
+    'vserver': VSERVER1,
+    'peer-vserver': VSERVER2,
+    'peer-state': 'peered',
+    'peer-cluster': 'fake_cluster'
+}]
+
 SNAPSHOT = {
     'id': SNAPSHOT_ID,
     'project_id': TENANT_ID,
@@ -508,6 +722,8 @@ COLLATED_CGSNAPSHOT_INFO = [
     },
 ]
 
+IDENTIFIER = 'c5b3a865-56d0-4d88-dke5-853465e099c9'
+
 LIF_NAMES = []
 LIF_ADDRESSES = ['10.10.10.10', '10.10.10.20']
 LIFS = (
@@ -637,8 +853,8 @@ POOLS = [
         'total_capacity_gb': 3.3,
         'free_capacity_gb': 1.1,
         'allocated_capacity_gb': 2.2,
-        'qos': 'False',
         'reserved_percentage': 5,
+        'max_over_subscription_ratio': 2.0,
         'dedupe': [True, False],
         'compression': [True, False],
         'thin_provisioning': [True, False],
@@ -660,8 +876,8 @@ POOLS = [
         'total_capacity_gb': 6.0,
         'free_capacity_gb': 2.0,
         'allocated_capacity_gb': 4.0,
-        'qos': 'False',
         'reserved_percentage': 5,
+        'max_over_subscription_ratio': 2.0,
         'dedupe': [True, False],
         'compression': [True, False],
         'thin_provisioning': [True, False],
@@ -686,8 +902,8 @@ POOLS_VSERVER_CREDS = [
         'total_capacity_gb': 'unknown',
         'free_capacity_gb': 1.1,
         'allocated_capacity_gb': 0.0,
-        'qos': 'False',
         'reserved_percentage': 5,
+        'max_over_subscription_ratio': 2.0,
         'dedupe': [True, False],
         'compression': [True, False],
         'thin_provisioning': [True, False],
@@ -706,8 +922,8 @@ POOLS_VSERVER_CREDS = [
         'total_capacity_gb': 'unknown',
         'free_capacity_gb': 2.0,
         'allocated_capacity_gb': 0.0,
-        'qos': 'False',
         'reserved_percentage': 5,
+        'max_over_subscription_ratio': 2.0,
         'dedupe': [True, False],
         'compression': [True, False],
         'thin_provisioning': [True, False],
@@ -1162,6 +1378,11 @@ PROCESSOR_INSTANCE_UUIDS = [
 ]
 PROCESSOR_INSTANCE_NAMES = ['processor0', 'processor1']
 
+SERVER_METADATA = {
+    'share_type_id': 'fake_id',
+    'host': 'fake_host',
+}
+
 PROCESSOR_COUNTERS = [
     {
         'node-name': 'cluster1-01',
@@ -1274,11 +1495,106 @@ EXPANDED_PROCESSOR_COUNTERS = [
     },
 ]
 
+SERVER_MIGRATION_CHECK_NOT_COMPATIBLE = {
+    'compatible': False,
+    'writable': None,
+    'nondisruptive': None,
+    'preserve_snapshots': None,
+    'migration_cancel': None,
+    'migration_get_progress': None,
+    'share_network_id': None
+}
+
+CIFS_SECURITY_SERVICE = {
+    'id': 'fake_id',
+    'type': 'active_directory',
+    'password': 'fake_password',
+    'user': 'fake_user',
+    'ou': 'fake_ou',
+    'domain': 'fake_domain',
+    'dns_ip': 'fake_dns_ip',
+    'server': '',
+}
+
+SHARE_NETWORK_SUBNET = {
+    'id': 'fake_share_net_subnet_d',
+    'neutron_subnet_id': '34950f50-a142-4328-8026-418ad4410b09',
+    'neutron_net_id': 'fa202676-531a-4446-bc0c-bcec15a72e82',
+    'network_type': 'fake_network_type',
+    'segmentation_id': 1234,
+    'ip_version': 4,
+    'cidr': 'fake_cidr',
+    'gateway': 'fake_gateway',
+    'mtu': 1509,
+}
+
+SHARE_NETWORK = {
+    'id': 'fake_share_net_id',
+    'project_id': 'fake_project_id',
+    'status': 'fake_status',
+    'name': 'fake_name',
+    'description': 'fake_description',
+    'security_services': [CIFS_SECURITY_SERVICE],
+    'subnets': [SHARE_NETWORK_SUBNET],
+}
+
+SHARE_TYPE_2 = copy.deepcopy(SHARE_TYPE)
+SHARE_TYPE_2['id'] = SHARE_TYPE_ID_2
+SHARE_TYPE_2['extra_specs'].update(SHARE_TYPE_EXTRA_SPEC)
+
+SHARE_REQ_SPEC = {
+    'share_properties': {
+        'size': SHARE['size'],
+        'project_id': SHARE['project_id'],
+        'snapshot_support': SHARE_TYPE_EXTRA_SPEC['snapshot_support'],
+        'create_share_from_snapshot_support':
+            SHARE_TYPE_EXTRA_SPEC['create_share_from_snapshot_support'],
+        'revert_to_snapshot_support':
+            SHARE_TYPE_EXTRA_SPEC['revert_to_snapshot_support'],
+        'mount_snapshot_support':
+            SHARE_TYPE_EXTRA_SPEC['mount_snapshot_support'],
+        'share_proto': SHARE['share_proto'],
+        'share_type_id': SHARE_TYPE_2['id'],
+        'is_public': True,
+        'share_group_id': None,
+        'source_share_group_snapshot_member_id': None,
+        'snapshot_id': None,
+    },
+    'share_instance_properties': {
+        'availability_zone_id': 'fake_az_1',
+        'share_network_id': SHARE_NETWORK['id'],
+        'share_server_id': SHARE_SERVER['id'],
+        'share_id': SHARE_ID,
+        'host': SHARE_INSTANCE['host'],
+        'status': SHARE_INSTANCE['status'],
+    },
+    'share_type': SHARE_TYPE_2,
+    'share_id': SHARE_ID,
+}
+
+SERVER_MIGRATION_REQUEST_SPEC = {
+    'shares_size': 10,
+    'snapshots_size': 10,
+    'shares_req_spec': [SHARE_REQ_SPEC],
+}
+
+CLIENT_GET_VOLUME_RESPONSE = {
+    'aggregate': AGGREGATE,
+    'junction-path': '/%s' % SHARE_NAME,
+    'name': SHARE_NAME,
+    'type': 'rw',
+    'style': 'flex',
+    'size': SHARE_SIZE,
+    'owning-vserver-name': VSERVER1,
+    'qos-policy-group-name': QOS_POLICY_GROUP_NAME,
+}
+
 
 def get_config_cmode():
     config = na_fakes.create_configuration_cmode()
     config.local_conf.set_override('share_backend_name', BACKEND_NAME)
     config.reserved_share_percentage = 5
+    config.max_over_subscription_ratio = 2.0
     config.netapp_login = CLIENT_KWARGS['username']
     config.netapp_password = CLIENT_KWARGS['password']
     config.netapp_server_hostname = CLIENT_KWARGS['hostname']
