@@ -18,9 +18,9 @@
 
 import os
 import time
+from unittest import mock
 
 import ddt
-import mock
 from oslo_concurrency import processutils
 from oslo_config import cfg
 from six import moves
@@ -323,6 +323,7 @@ class GenericShareDriverTestCase(test.TestCase):
                 '&&', 'sudo', 'mount', volume['mountpoint'], mount_path,
                 '&&', 'sudo', 'chmod', '777', mount_path,
                 '&&', 'sudo', 'umount', mount_path,
+                '&&', 'sudo', 'e2fsck', '-y', '-f', volume['mountpoint'],
                 '&&', 'sudo', 'tune2fs', '-U', 'random', volume['mountpoint'],
                 '&&', 'sudo', 'mount', volume['mountpoint'], mount_path,
             ),
@@ -491,13 +492,13 @@ class GenericShareDriverTestCase(test.TestCase):
     def test_add_mount_permanently(self):
         self.mock_object(self._driver, '_ssh_exec')
         self._driver._add_mount_permanently(self.share.id, self.server)
-        self._driver._ssh_exec.has_calls(
+        self._driver._ssh_exec.assert_has_calls([
             mock.call(
                 self.server,
                 ['grep', self.share.id, const.MOUNT_FILE_TEMP,
                  '|', 'sudo', 'tee', '-a', const.MOUNT_FILE]),
             mock.call(self.server, ['sudo', 'mount', '-a'])
-        )
+        ])
 
     def test_add_mount_permanently_raise_error_on_add(self):
         self.mock_object(
