@@ -15,9 +15,9 @@
 """Unit tests for the LVM driver module."""
 
 import os
+from unittest import mock
 
 import ddt
-import mock
 from oslo_concurrency import processutils
 from oslo_config import cfg
 from oslo_utils import timeutils
@@ -150,16 +150,6 @@ class LVMShareDriverTestCase(test.TestCase):
         self.assertRaises(exception.InvalidParameterValue,
                           self._driver.check_for_setup_error)
 
-    def test_check_for_setup_error_deprecated_export_ip(self):
-        def exec_runner(*ignore_args, **ignore_kwargs):
-            return '\n   fake1\n   fakevg\n   fake2\n', ''
-
-        fake_utils.fake_execute_set_repliers([('vgs --noheadings -o name',
-                                               exec_runner)])
-        CONF.set_default('lvm_share_export_ip', CONF.lvm_share_export_ips[0])
-        CONF.set_default('lvm_share_export_ips', None)
-        self.assertIsNone(self._driver.check_for_setup_error())
-
     def test_check_for_setup_error_no_export_ips(self):
         def exec_runner(*ignore_args, **ignore_kwargs):
             return '\n   fake1\n   fakevg\n   fake2\n', ''
@@ -170,16 +160,6 @@ class LVMShareDriverTestCase(test.TestCase):
         self.assertRaises(exception.InvalidParameterValue,
                           self._driver.check_for_setup_error)
 
-    def test_check_for_setup_error_both_export_ip_and_ips(self):
-        def exec_runner(*ignore_args, **ignore_kwargs):
-            return '\n   fake1\n   fakevg\n   fake2\n', ''
-
-        fake_utils.fake_execute_set_repliers([('vgs --noheadings -o name',
-                                               exec_runner)])
-        CONF.set_default('lvm_share_export_ip', CONF.lvm_share_export_ips[0])
-        self.assertRaises(exception.InvalidParameterValue,
-                          self._driver.check_for_setup_error)
-
     def test_local_path_normal(self):
         share = fake_share(name='fake_sharename')
         CONF.set_default('lvm_share_volume_group', 'fake_vg')
@@ -228,6 +208,7 @@ class LVMShareDriverTestCase(test.TestCase):
         expected_exec = [
             'lvcreate -L 1G -n fakename fakevg',
             'mkfs.ext4 /dev/mapper/fakevg-fakename',
+            'e2fsck -y -f %s' % mount_share,
             'tune2fs -U random %s' % mount_share,
             ("dd count=0 if=%s of=%s iflag=direct oflag=direct" %
              (mount_snapshot, mount_share)),
@@ -267,9 +248,12 @@ class LVMShareDriverTestCase(test.TestCase):
                           self._driver._deallocate_container,
                           self.share['name'])
 
-    def test_deallocate_container_not_found_error(self):
+    @ddt.data(
+        'Logical volume "fake/fake-volume" not found\n',
+        'Failed to find logical volume "fake/fake-volume"\n')
+    def test_deallocate_container_not_found_error(self, error_msg):
         def _fake_exec(*args, **kwargs):
-            raise exception.ProcessExecutionError(stderr="not found")
+            raise exception.ProcessExecutionError(stderr=error_msg)
 
         self.mock_object(self._driver, '_try_execute', _fake_exec)
         self._driver._deallocate_container(self.share['name'])
@@ -287,6 +271,22 @@ class LVMShareDriverTestCase(test.TestCase):
                              self._driver.get_share_stats(refresh=True))
         self._driver._update_share_stats.assert_called_once_with()
 
+    def test__unmount_device_not_mounted(self):
+        def exec_runner(*ignore_args, **ignore_kwargs):
+            umount_msg = (
+                "umount: /opt/stack/data/manila/mnt/share-fake-share: not "
+                "mounted.\n"
+            )
+            raise exception.ProcessExecutionError(stderr=umount_msg)
+        self._os.path.exists.return_value = True
+        mount_path = self._get_mount_path(self.share)
+        expected_exec = "umount -f %s" % (mount_path)
+        fake_utils.fake_execute_set_repliers([(expected_exec, exec_runner)])
+
+        self._driver._unmount_device(self.share, raise_if_missing=False)
+
+        self._os.path.exists.assert_called_with(mount_path)
+
     def test__unmount_device_is_busy_error(self):
         def exec_runner(*ignore_args, **ignore_kwargs):
             raise exception.ProcessExecutionError(stderr='device is busy')
@@ -333,6 +333,7 @@ class LVMShareDriverTestCase(test.TestCase):
         expected_exec = [
             ("lvcreate -L 1G --name fakesnapshotname --snapshot "
              "%s/fakename" % (CONF.lvm_share_volume_group,)),
+            "e2fsck -y -f /dev/mapper/fakevg-%s" % self.snapshot['name'],
             "tune2fs -U random /dev/mapper/fakevg-%s" % self.snapshot['name'],
             "mkdir -p " + mount_path,
             "mount /dev/mapper/fakevg-fakesnapshotname " + mount_path,
@@ -401,15 +402,15 @@ class LVMShareDriverTestCase(test.TestCase):
                 self.server, self.share['name'],
                 access_rules, add_rules=add_rules, delete_rules=delete_rules))
 
-    @ddt.data(('1001::1001/129', None, False), ('1.1.1.256', None, False),
-              ('1001::1001', None, [6]), ('1.1.1.0', None, [4]),
-              (None, ['1001::1001', '1.1.1.0'], [6, 4]),
-              (None, ['1001::1001'], [6]), (None, ['1.1.1.0'], [4]),
-              (None, ['1001::1001/129', '1.1.1.0'], False))
+    @ddt.data((['1001::1001/129'], False),
+              (['1.1.1.256'], False),
+              (['1001::1001'], [6]),
+              ('1.1.1.0', [4]),
+              (['1001::1001', '1.1.1.0'], [6, 4]),
+              (['1001::1001/129', '1.1.1.0'], False))
     @ddt.unpack
-    def test_get_configured_ip_versions(
-            self, configured_ip, configured_ips, configured_ip_version):
-        CONF.set_default('lvm_share_export_ip', configured_ip)
+    def test_get_configured_ip_versions(self, configured_ips,
+                                        configured_ip_version):
         CONF.set_default('lvm_share_export_ips', configured_ips)
         if configured_ip_version:
             self.assertEqual(configured_ip_version,
@@ -485,8 +486,6 @@ class LVMShareDriverTestCase(test.TestCase):
         self._driver.extend_share(self.share, 3)
         self._driver._extend_container.assert_called_once_with(self.share,
                                                                local_path, 3)
-        self._driver._execute.assert_called_once_with('resize2fs', local_path,
-                                                      run_as_root=True)
 
     def test_ssh_exec_as_root(self):
         command = ['fake_command']
@@ -509,7 +508,7 @@ class LVMShareDriverTestCase(test.TestCase):
             'lvextend',
             '-L',
             '3G',
-            '-n',
+            '-r',
             'device_name',
             run_as_root=True)
 
@@ -545,10 +544,10 @@ class LVMShareDriverTestCase(test.TestCase):
                                               'count=1024', 'bs=1M',
                                               run_as_root=True)
 
-    @ddt.data(('1.1.1.1', 4), ('1001::1001', 6))
+    @ddt.data((['1.1.1.1'], 4), (['1001::1001'], 6))
     @ddt.unpack
     def test_update_share_stats(self, configured_ip, version):
-        CONF.set_default('lvm_share_export_ip', configured_ip)
+        CONF.set_default('lvm_share_export_ips', configured_ip)
         self.mock_object(self._driver, 'get_share_server_pools',
                          mock.Mock(return_value='test-pool'))
 
@@ -579,7 +578,9 @@ class LVMShareDriverTestCase(test.TestCase):
             ("lvconvert --merge %s" % snap_lv),
             ("lvcreate -L 1G --name fakesnapshotname --snapshot %s" %
                 share_lv),
-            ('tune2fs -U random /dev/mapper/%s-fakesnapshotname' %
+            ("e2fsck -y -f /dev/mapper/%s-fakesnapshotname" %
+                CONF.lvm_share_volume_group),
+            ("tune2fs -U random /dev/mapper/%s-fakesnapshotname" %
                 CONF.lvm_share_volume_group),
             ("mkdir -p %s" % share_mount_path),
             ("mount /dev/mapper/%s-fakename %s" %
