@@ -13,8 +13,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from unittest import mock
+
 import ddt
-import mock
 from oslo_config import cfg
 
 from manila import exception
@@ -353,6 +354,7 @@ class DbQuotaDriverTestCase(test.TestCase):
             'project_id': self.ctxt.project_id,
             'user_id': self.ctxt.user_id,
             'share_type_id': None,
+            'overquota_allowed': False
         }
         expected_kwargs.update(kwargs)
         st_quotas = st_quotas if kwargs.get('share_type_id') else {}
@@ -440,14 +442,14 @@ class DbQuotaDriverTestCase(test.TestCase):
 
     def test_destroy_all_by_project_and_share_type(self):
         mock_destroy_all = self.mock_object(
-            quota.db, 'quota_destroy_all_by_project_and_share_type')
+            quota.db, 'quota_destroy_all_by_share_type')
 
         result = self.driver.destroy_all_by_project_and_share_type(
             self.ctxt, self.project_id, self.share_type_id)
 
         self.assertIsNone(result)
         mock_destroy_all.assert_called_once_with(
-            self.ctxt, self.project_id, self.share_type_id)
+            self.ctxt, self.share_type_id, project_id=self.project_id)
 
     def test_expire(self):
         self.mock_object(quota.db, 'reservation_expire')
@@ -624,7 +626,8 @@ class QuotaEngineTestCase(test.TestCase):
         self.driver.reserve.assert_called_once_with(
             self.ctxt, self.engine._resources, {'delta1': 1, 'delta2': 2},
             expire='fake_expire', project_id=self.project_id,
-            user_id=self.user_id, share_type_id=self.share_type_id)
+            user_id=self.user_id, share_type_id=self.share_type_id,
+            overquota_allowed=False)
 
     @ddt.data(Exception('FakeException'), [None])
     def test_commit(self, side_effect):
@@ -711,6 +714,7 @@ class QuotaEngineTestCase(test.TestCase):
 
     def test_current_common_resources(self):
         self.assertEqual(
-            ['gigabytes', 'share_group_snapshots', 'share_groups',
-             'share_networks', 'shares', 'snapshot_gigabytes', 'snapshots'],
+            ['gigabytes', 'replica_gigabytes', 'share_group_snapshots',
+             'share_groups', 'share_networks', 'share_replicas', 'shares',
+             'snapshot_gigabytes', 'snapshots'],
             quota.QUOTAS.resources)
