@@ -12,9 +12,10 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from unittest import mock
+
 from cinderclient import exceptions as cinder_exception
 import ddt
-import mock
 
 from manila import context
 from manila import exception
@@ -55,8 +56,6 @@ class CinderclientTestCase(test.TestCase):
         fake_context = 'fake_context'
         data = {
             'cinder': {
-                'api_insecure': True,
-                'ca_certificates_file': 'foo_ca_certificates_file',
                 'http_retries': 3,
                 'endpoint_type': 'foo_endpoint_type',
                 'region_name': 'foo_region_name',
@@ -73,8 +72,6 @@ class CinderclientTestCase(test.TestCase):
         )
         mock_client_loader.return_value.get_client.assert_called_once_with(
             fake_context,
-            insecure=data['cinder']['api_insecure'],
-            cacert=data['cinder']['ca_certificates_file'],
             retries=data['cinder']['http_retries'],
             endpoint_type=data['cinder']['endpoint_type'],
             region_name=data['cinder']['region_name'],
@@ -85,8 +82,6 @@ class CinderclientTestCase(test.TestCase):
         fake_context = 'fake_context'
         data = {
             'cinder': {
-                'api_insecure': True,
-                'ca_certificates_file': 'foo_ca_certificates_file',
                 'http_retries': 3,
                 'endpoint_type': 'foo_endpoint_type',
                 'region_name': 'foo_region_name',
@@ -98,8 +93,6 @@ class CinderclientTestCase(test.TestCase):
 
         cinder.AUTH_OBJ.get_client.assert_called_once_with(
             fake_context,
-            insecure=data['cinder']['api_insecure'],
-            cacert=data['cinder']['ca_certificates_file'],
             retries=data['cinder']['http_retries'],
             endpoint_type=data['cinder']['endpoint_type'],
             region_name=data['cinder']['region_name'],
